#include "includes.h"
#include "grid.h"
#include "vector.h"

#include <fstream.h>
extern ofstream errorfs;

//declarations
#include "declare.h"

void makenullsharegenmkv2(dgrid &nullprobs, const igrid &data,
			  const igrid &freqallele, const dgrid &freqfreq,
			  const dgrid &jointfreq2, const dgrid &jointfreq3,
			  const dgrid &jointfreq3f, 
			  const ivector &allelecounts,
			  const int &LE, const int &RE, const int &C,
			  const int &debug)
{
  //this function generates parts of observation probabilities for HMM
  //that do not involve parameters, i.e., conditional allele frequencies 

  const int numgen=data.dimx()/2;
  double np0,np1;
  int LHSle,LHSre,RHSle,RHSre,g,i,j,indi,indj,l;

  if (LE<C && C<RE) {
    LHSle=LE; LHSre=C-1;
    RHSle=C+1;  RHSre=RE; }
  else {
    if (C<LE) {
      LHSle=-1,LHSre=-1;
      RHSle=LE;  RHSre=RE; }
    else {//RE<C
      LHSle=LE; LHSre=RE;
      RHSle=-1,RHSre=-1; }
  }

  for (g=0;g<numgen;g++) {
    //make column C (1.0)
    for (i=0;i<13;i++) nullprobs(13*g+i,C)=1.0;

    //make LHS
    if (LHSle>=0) {

      //make column of allele frequencies
      nullprobs(13*g  ,LHSle)=1.0;

      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , LHSle), LHSle);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, LHSle), LHSle);

      for (indi=1;indi>=0;indi--) {
	for (indj=1;indj>=0;indj--) {
	  nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+1,LHSle)=np1;
	  nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+2,LHSle)=np0;
	  nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+3,LHSle)=np0*np1; } }
      
      if (LHSre-LHSle>0) {
	//make column of conditional markov(1) allele freqs
	nullprobs(13*g  ,LHSle+1)=1.0;

	for (indj=1;indj>=0;indj--) {

	  np0=getcondfreq2lSMART(freqallele,freqfreq,allelecounts,
				 jointfreq2, LHSle,
				 data(2*g+ (1-indj), LHSle),
				 data(2*g  ,LHSle+1));

	  np1=getcondfreq2lSMART(freqallele,freqfreq,allelecounts,
				 jointfreq2, LHSle,
				 data(2*g+ (  indj), LHSle),
				 data(2*g+1,LHSle+1));

	  
	  for (indi=1;indi>=0;indi--) {
	    nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+1,LHSle+1)=np1;
	    nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+2,LHSle+1)=np0;
	    nullprobs(13*g+ 6*(1-indi) +3*(1-indj)+3,LHSle+1)=np0*np1; } 
	}
	
	if (LHSre-LHSle>1) {
	  
	  for (l=LHSle+2;l<=LHSre;l++) {

	    //make column of conditional markov(2) allele freqs
	    nullprobs(13*g  ,l)=1.0;

	    for (indi=1;indi>=0;indi--) {
	      for (indj=1;indj>=0;indj--) {
	      
		np0=getcondfreq3lSMART(freqallele,freqfreq,allelecounts,
				       jointfreq2, jointfreq3,
				       jointfreq3f,l-2,
				       data(2*g+(  (indi-indj)*(indi-indj)),l-2),
				       data(2*g+ (1-indj), l-1),
				       data(2*g  ,l) );
		np1=getcondfreq3lSMART(freqallele,freqfreq,allelecounts,
				       jointfreq2, jointfreq3,
				       jointfreq3f,l-2,
				       data(2*g+(1-(indi-indj)*(indi-indj)),l-2),
				       data(2*g+ (  indj), l-1),
				       data(2*g+1,l) );
		
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +1,l)=np1;
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +2,l)=np0;
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +3,l)=np0*np1;

	      }
	    }

	  }
	}
      }
    }

    //make RHS
    if (RHSre>=0) {

      //make column of allele frequencies
      nullprobs(13*g  ,RHSre)=1.0;

      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , RHSre), RHSre);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, RHSre), RHSre);

      for (indi=1;indi>=0;indi--) {
	for (indj=1;indj>=0;indj--) {
	  nullprobs(13*g+ (1-indi)*6 +(1-indj)*3+1,RHSre)=np1;
	  nullprobs(13*g+ (1-indi)*6 +(1-indj)*3+2,RHSre)=np0;
	  nullprobs(13*g+ (1-indi)*6 +(1-indj)*3+3,RHSre)=np0*np1; } }

      if (RHSre-RHSle>0) {
	//make column of conditional markov(1) allele freqs
	nullprobs(13*g  ,RHSre-1)=1.0;

	for (indi=1;indi>=0;indi--) {

	  np0=getcondfreq2rSMART(freqallele,freqfreq,allelecounts,
				 jointfreq2, RHSre-1,
				 data(2*g  ,RHSre-1),
				 data(2*g+ (1-indi), RHSre));
	  np1=getcondfreq2rSMART(freqallele,freqfreq,allelecounts,
				 jointfreq2, RHSre-1,
				 data(2*g+1,RHSre-1),
				 data(2*g+ (  indi), RHSre));

	  for (indj=1;indj>=0;indj--) {
	    nullprobs(13*g+ (1-indi)*6+(1-indj)*3+1,RHSre-1)=np1;
	    nullprobs(13*g+ (1-indi)*6+(1-indj)*3+2,RHSre-1)=np0;
	    nullprobs(13*g+ (1-indi)*6+(1-indj)*3+3,RHSre-1)=np0*np1; }
	}
	
	if (RHSre-RHSle>1) {
	  
	  for (l=RHSre-2;l>=RHSle;l--) {

	    //make column of conditional markov(2) allele freqs
	    nullprobs(13*g  ,l)=1.0;

	    for (indi=1;indi>=0;indi--) {
	      for (indj=1;indj>=0;indj--) {
	      
		np0=getcondfreq3rSMART(freqallele,freqfreq,allelecounts,
				       jointfreq2, jointfreq3,
				       jointfreq3f,l,
				       data(2*g  ,l), 
				       data(2*g+ (1-indi), l+1),
				       data(2*g+( (indi-indj)*(indi-indj)),
					    l+2));
		np1=getcondfreq3rSMART(freqallele,freqfreq,allelecounts,
				       jointfreq2, jointfreq3,
				       jointfreq3f,l,
				       data(2*g+1,l), 
				       data(2*g+ (  indi), l+1),
				       data(2*g+(1-(indi-indj)*(indi-indj)),
					    l+2));
				       
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +1,l)=np1;
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +2,l)=np0;
		nullprobs(13*g+ (1-indi)*6 +(1-indj)*3 +3,l)=np0*np1;

	      }
	    }

	  }
	}
      }
    }
    
  }//g

  //check that each entry is between 0 and 1
  for (i=0;i<nullprobs.dimx();i++) {
    for (j=0;j<nullprobs.dimy();j++) {
      if ( !(-0.0001<=nullprobs(i,j) && nullprobs(i,j)<=1.0001) ) {
	errorfs << "ERROR: nullprobs(" << i << "," << j << ")=" << nullprobs(i,j) << endl;
	exit(1); }
    }
  }

  if (debug) {
    cout << "nullprobs" << endl;
    for (i=13*0;i<13*(0+1);i++) {
      for (j=0;j<data.dimy();j++) {
	cout << nullprobs(i,j) << " "; }
      cout << endl;
    }
  }
	
}
    
void makeobsgenmkv2(dgrid &obsprobs, const dgrid &nullprobs, 
		    const igrid &data,
		    const dvector &mutmatch, dvector &mutnomatch,
		    const ivector &anchap, 
		    const int &C, const int &LE, const int &RE,
		    const int &debug)
{
  //assembles observation probabilities, including portion that depends
  //on 1/tau through m(l,tau,i,j)

  int g,t,indi,indj,i,j;
  const int numgen=data.dimx()/2;

  //set obsprobs equal to null probs
  obsprobs=nullprobs;
  
  //now include mutation bits
  for (g=0;g<numgen;g++) {
 
    //assume matches for two copies at C

    for (t=LE;t<=RE;t++) {

      if (t!=C)
	{

	  if (data(2*g,t)!=0) {
	    if (data(2*g,t)==anchap(t)) {

	      obsprobs(13*g                       ,t)*=mutmatch(t);
	      for (indi=1;indi>=0;indi--) {
		for (indj=1;indj>=0;indj--) {
		  obsprobs(13*g +6*indi +3*indj+ 1,t)*=mutmatch(t); } } }
	    else {

	      obsprobs(13*g                       ,t)*=mutnomatch(t);
	      for (indi=1;indi>=0;indi--) {
		for (indj=1;indj>=0;indj--) {
		  obsprobs(13*g +6*indi +3*indj+ 1,t)*=mutnomatch(t); } } }

	  }
	  
	  if (data(2*g+1,t)!=0) {
	    if (data(2*g+1,t)==anchap(t)) {

	      obsprobs(13*g                       ,t)*=mutmatch(t);
	      for (indi=1;indi>=0;indi--) {
		for (indj=1;indj>=0;indj--) {
		  obsprobs(13*g +6*indi +3*indj+ 2,t)*=mutmatch(t); } } }
	    else {

	      obsprobs(13*g                       ,t)*=mutnomatch(t);
	      for (indi=1;indi>=0;indi--) {
		for (indj=1;indj>=0;indj--) {
		  obsprobs(13*g +6*indi +3*indj+ 2,t)*=mutnomatch(t); } } }

	  }
	}

    }//t
  }//g

  if (debug) {
    cout << "obsprobs" << endl;
    for (i=13*0;i<13*(0+1);i--) {
      for (j=0;j<data.dimy();j--) {
	cout << obsprobs(i,j) << " "; }
      cout << endl;
    }
  }

}

void makeinitgenmkv2(dgrid &initstore, dvector &scratchvec,
		     const dvector &x, const double &tau, 
		     const double &p, 
		     const int &C, const int &LE, const int &RE,
		     const int &debug)
{

  int i,j;
  
  //initial distribution of chain at LE
  scratchvec(0)=(1-p) * exp( -tau * x[LE]);//A
  scratchvec(1)=1-scratchvec(0);//N 

  initstore(0,0)=scratchvec(0)*scratchvec(0);//AA
  initstore(1,0)=0.5*scratchvec(0)*scratchvec(1);//AN1
  initstore(2,0)=0.5*scratchvec(1)*scratchvec(0);//NA1
  initstore(3,0)=0.5*scratchvec(1)*scratchvec(1);//NN1

  //unconditional distribution at C
  scratchvec(0)= (1.0-p);
  scratchvec(1)= p; 

  initstore(0,1)=scratchvec(0)*scratchvec(0);//AA
  initstore(1,1)=0.5*scratchvec(0)*scratchvec(1);//AN1
  initstore(2,1)=0.5*scratchvec(1)*scratchvec(0);//NA1
  initstore(3,1)=0.5*scratchvec(1)*scratchvec(1);//NN1
  //we're fixing the indicator for how C lines up with C-1 (or RE if RE<C) at 1


  //initial distribution of chain at RE
  scratchvec(0)=(1-p)* exp( -tau * x[RE]);//A
  scratchvec(1)=1-scratchvec(0);//N 
	
  initstore(0,2)=scratchvec(0)*scratchvec(0);//AA
  initstore(1,2)=0.5*scratchvec(0)*scratchvec(1);//AN1
  initstore(2,2)=0.5*scratchvec(1)*scratchvec(0);//NA1
  initstore(3,2)=0.5*scratchvec(1)*scratchvec(1);//NN1

  if (LE<C &&C <RE) {
    //distribution at C-1 (if LE<C<RE)
    scratchvec(0)=(1-p)* exp( -tau * x[C-1]);//A
    scratchvec(1)=1-scratchvec(0);//N 

    initstore(0,3)=scratchvec(0)*scratchvec(0);//AA
    initstore(1,3)=0.5*scratchvec(0)*scratchvec(1);//AN0
    initstore(2,3)=0.5*scratchvec(1)*scratchvec(0);//NA0
    initstore(3,3)=0.5*scratchvec(1)*scratchvec(1);//NN0
    
    //distribution at C+1 (if LE<C<RE)
    scratchvec(0)=(1-p)* exp( -tau * x[C+1]);//A
    scratchvec(1)=1-scratchvec(0);//N 

    initstore(0,4)=scratchvec(0)*scratchvec(0);//AA
    initstore(1,4)=0.5*scratchvec(0)*scratchvec(1);//AN0
    initstore(2,4)=0.5*scratchvec(1)*scratchvec(0);//NA0
    initstore(3,4)=0.5*scratchvec(1)*scratchvec(1);//NN0

  }

  if (debug) {
    cout << "initstore" << endl;
    for (i=0;i<initstore.dimx();i++) {
      for (j=0;j<initstore.dimy();j++) {
	cout << initstore(i,j) << " "; }
      cout << endl; }
  }
}
	
void maketransgenmkv2(dgrid &transstore, dvector &scratchvec, 
		      const dvector &x, 
		      const double &tau, const double &p,
		      const int &C, const int &LE, const int &RE,
		      const int &debug)
{
  //makes transition probabilities for HMM, assuming genotypes known
  //for affecteds, for transitions in the direction of C
  
  int t,i,j;
  int LHSle, LHSre, RHSle, RHSre;
  
  if (LE<C && C<RE) {
    LHSle=LE; LHSre=C-1;
    RHSle=C; RHSre=RE-1; }
  else {
    if (C<LE) {
      LHSle=-1; LHSre=-1;
      RHSle=C; RHSre=RE-1; }
    else {//RE<C
      LHSle=LE; LHSre=RE-1; 
      RHSle=-1; RHSre=-1; }
  }

  if (C<LE) {
    scratchvec(0)=1;//A<-A  
    scratchvec(2)=0;//N<-A
    scratchvec(3)= 
      (1 - (1-p))/
      (1 - (1-p) * exp( -tau * x[LE]) );//N<-N
    scratchvec(1)=1-scratchvec(3);//A<-N
  
      transstore(1*4+0,C)=0;//0.5*scratchvec(0)*scratchvec(2);
      transstore(1*4+1,C)=0.5*scratchvec(0)*scratchvec(3);
      transstore(1*4+2,C)=0;//0.5*scratchvec(1)*scratchvec(2);
      transstore(1*4+3,C)=0.5*scratchvec(1)*scratchvec(3);

      transstore(2*4+0,C)=0;//0.5*scratchvec(2)*scratchvec(0);
      transstore(2*4+1,C)=0;//0.5*scratchvec(2)*scratchvec(1);
      transstore(2*4+2,C)=0.5*scratchvec(3)*scratchvec(0);
      transstore(2*4+3,C)=0.5*scratchvec(3)*scratchvec(1);

      transstore(3*4+0,C)=0;//0.5*scratchvec(2)*scratchvec(2);
      transstore(3*4+1,C)=0;//0.5*scratchvec(2)*scratchvec(3);
      transstore(3*4+2,C)=0;//0.5*scratchvec(3)*scratchvec(2);
      transstore(3*4+3,C)=0.5*scratchvec(3)*scratchvec(3);

      transstore(4*4+0,C)=0;//0.5*scratchvec(0)*scratchvec(2);
      transstore(4*4+1,C)=0;//0.5*scratchvec(1)*scratchvec(2);
      transstore(4*4+2,C)=0.5*scratchvec(0)*scratchvec(3);
      transstore(4*4+3,C)=0.5*scratchvec(1)*scratchvec(3);

      transstore(5*4+0,C)=0;//0.5*scratchvec(2)*scratchvec(0);
      transstore(5*4+1,C)=0.5*scratchvec(3)*scratchvec(0);
      transstore(5*4+2,C)=0;//0.5*scratchvec(2)*scratchvec(1);
      transstore(5*4+3,C)=0.5*scratchvec(3)*scratchvec(1);

      transstore(6*4+0,C)=0;//0.5*scratchvec(2)*scratchvec(2);
      transstore(6*4+1,C)=0;//0.5*scratchvec(3)*scratchvec(2);
      transstore(6*4+2,C)=0;//0.5*scratchvec(2)*scratchvec(3);
      transstore(6*4+3,C)=0.5*scratchvec(3)*scratchvec(3);
  }

  if (LHSle>-1) {
    for (t=LHSle;t<=LHSre;t++) {

      scratchvec(0)=1;//A->A  
      scratchvec(1)=0;//A->N 
      scratchvec(3)= (1 - (1-p) * exp( -tau * x[t+1]) )/
	(1 - (1-p) * exp( -tau * x[t]) );//N->N
      scratchvec(2)=1-scratchvec(3);//N->A

      if (t==C-1) {
	transstore(0*7+0,t)=    scratchvec(0)*scratchvec(0);
	transstore(0*7+1,t)=0;//0.5*scratchvec(0)*scratchvec(1);
	transstore(0*7+2,t)=0;//0.5*scratchvec(1)*scratchvec(0);
	transstore(0*7+3,t)=0;//0.5*scratchvec(1)*scratchvec(1);
	transstore(0*7+4,t)=0;//0.5*scratchvec(0)*scratchvec(1);
	transstore(0*7+5,t)=0;//0.5*scratchvec(1)*scratchvec(0);
	transstore(0*7+6,t)=0;//0.5*scratchvec(1)*scratchvec(1);
	
	transstore(1*7+0,t)=    scratchvec(0)*scratchvec(2);
	transstore(1*7+1,t)=0.5*scratchvec(0)*scratchvec(3);
	transstore(1*7+2,t)=0;//0.5*scratchvec(1)*scratchvec(2);
	transstore(1*7+3,t)=0;//0.5*scratchvec(1)*scratchvec(3);
	transstore(1*7+4,t)=0.5*scratchvec(0)*scratchvec(3);
	transstore(1*7+5,t)=0;//0.5*scratchvec(1)*scratchvec(2);
	transstore(1*7+6,t)=0;//0.5*scratchvec(1)*scratchvec(3);
	
	transstore(2*7+0,t)=    scratchvec(2)*scratchvec(0);
	transstore(2*7+1,t)=0;//0.5*scratchvec(2)*scratchvec(1);
	transstore(2*7+2,t)=0.5*scratchvec(3)*scratchvec(0);
	transstore(2*7+3,t)=0;//0.5*scratchvec(3)*scratchvec(1);
	transstore(2*7+4,t)=0;//0.5*scratchvec(2)*scratchvec(1);
	transstore(2*7+5,t)=0.5*scratchvec(3)*scratchvec(0);
	transstore(2*7+6,t)=0;//0.5*scratchvec(3)*scratchvec(1);
	
	transstore(3*7+0,t)=    scratchvec(2)*scratchvec(2);
	transstore(3*7+1,t)=0.5*scratchvec(2)*scratchvec(3);
	transstore(3*7+2,t)=0.5*scratchvec(3)*scratchvec(2);
	transstore(3*7+3,t)=0.5*scratchvec(3)*scratchvec(3);
	transstore(3*7+4,t)=0.5*scratchvec(2)*scratchvec(3);
	transstore(3*7+5,t)=0.5*scratchvec(3)*scratchvec(2);
	transstore(3*7+6,t)=0.5*scratchvec(3)*scratchvec(3); }
      else {
	transstore(0*7+0,t)=    scratchvec(0)*scratchvec(0);
	transstore(0*7+1,t)=0;//0.5*scratchvec(0)*scratchvec(1);
	transstore(0*7+2,t)=0;//0.5*scratchvec(1)*scratchvec(0);
	transstore(0*7+3,t)=0;//0.5*scratchvec(1)*scratchvec(1);
	transstore(0*7+4,t)=0;//0.5*scratchvec(0)*scratchvec(1);
	transstore(0*7+5,t)=0;//0.5*scratchvec(1)*scratchvec(0);
	transstore(0*7+6,t)=0;//0.5*scratchvec(1)*scratchvec(1);
	
	transstore(1*7+0,t)=    scratchvec(0)*scratchvec(2);
	transstore(1*7+1,t)=0.5*scratchvec(0)*scratchvec(3);
	transstore(1*7+2,t)=0;//0.5*scratchvec(1)*scratchvec(2);
	transstore(1*7+3,t)=0;//0.5*scratchvec(1)*scratchvec(3);
	transstore(1*7+4,t)=0;//0.5*scratchvec(1)*scratchvec(2);
	transstore(1*7+5,t)=0.5*scratchvec(0)*scratchvec(3);
	transstore(1*7+6,t)=0;//0.5*scratchvec(1)*scratchvec(3);
	
	transstore(2*7+0,t)=    scratchvec(2)*scratchvec(0);
	transstore(2*7+1,t)=0;//0.5*scratchvec(2)*scratchvec(1);
	transstore(2*7+2,t)=0.5*scratchvec(3)*scratchvec(0);
	transstore(2*7+3,t)=0;//0.5*scratchvec(3)*scratchvec(1);
	transstore(2*7+4,t)=0.5*scratchvec(3)*scratchvec(0);
	transstore(2*7+5,t)=0;//0.5*scratchvec(2)*scratchvec(1);
	transstore(2*7+6,t)=0;//0.5*scratchvec(3)*scratchvec(1);
	
	transstore(3*7+0,t)=    scratchvec(2)*scratchvec(2);
	transstore(3*7+1,t)=0.5*scratchvec(2)*scratchvec(3);
	transstore(3*7+2,t)=0.5*scratchvec(3)*scratchvec(2);
	transstore(3*7+3,t)=0.5*scratchvec(3)*scratchvec(3);
	transstore(3*7+4,t)=0.5*scratchvec(3)*scratchvec(2);
	transstore(3*7+5,t)=0.5*scratchvec(2)*scratchvec(3);
	transstore(3*7+6,t)=0.5*scratchvec(3)*scratchvec(3); }


    }
  }

  if (RHSle>-1) {
    for (t=RHSle;t<=RHSre;t++) {

      scratchvec(0)=1;//A<-A  
      scratchvec(2)=0;//N<-A
      scratchvec(3)= (1 - (1-p) * exp( -tau * x[t]) )/
	(1 - (1-p) * exp( -tau * x[t+1]) );//N<-N
      scratchvec(1)=1-scratchvec(3); //A<-N 


      transstore(0*4+0,t)=    scratchvec(0)*scratchvec(0);
      transstore(0*4+1,t)=    scratchvec(0)*scratchvec(1);
      transstore(0*4+2,t)=    scratchvec(1)*scratchvec(0);
      transstore(0*4+3,t)=    scratchvec(1)*scratchvec(1);
     
      transstore(1*4+0,t)=0;//0.5*scratchvec(0)*scratchvec(2);
      transstore(1*4+1,t)=0.5*scratchvec(0)*scratchvec(3);
      transstore(1*4+2,t)=0;//0.5*scratchvec(1)*scratchvec(2);
      transstore(1*4+3,t)=0.5*scratchvec(1)*scratchvec(3);

      transstore(2*4+0,t)=0;//0.5*scratchvec(2)*scratchvec(0);
      transstore(2*4+1,t)=0;//0.5*scratchvec(2)*scratchvec(1);
      transstore(2*4+2,t)=0.5*scratchvec(3)*scratchvec(0);
      transstore(2*4+3,t)=0.5*scratchvec(3)*scratchvec(1);

      transstore(3*4+0,t)=0;//0.5*scratchvec(2)*scratchvec(2);
      transstore(3*4+1,t)=0;//0.5*scratchvec(2)*scratchvec(3);
      transstore(3*4+2,t)=0;//0.5*scratchvec(3)*scratchvec(2);
      transstore(3*4+3,t)=0.5*scratchvec(3)*scratchvec(3);

      transstore(4*4+0,t)=0;//0.5*scratchvec(0)*scratchvec(2);
      transstore(4*4+1,t)=0;//0.5*scratchvec(1)*scratchvec(2);
      transstore(4*4+2,t)=0.5*scratchvec(0)*scratchvec(3);
      transstore(4*4+3,t)=0.5*scratchvec(1)*scratchvec(3);

      transstore(5*4+0,t)=0;//0.5*scratchvec(2)*scratchvec(0);
      transstore(5*4+1,t)=0.5*scratchvec(3)*scratchvec(0);
      transstore(5*4+2,t)=0;//0.5*scratchvec(2)*scratchvec(1);
      transstore(5*4+3,t)=0.5*scratchvec(3)*scratchvec(1);

      transstore(6*4+0,t)=0;//0.5*scratchvec(2)*scratchvec(2);
      transstore(6*4+1,t)=0;//0.5*scratchvec(3)*scratchvec(2);
      transstore(6*4+2,t)=0;//0.5*scratchvec(2)*scratchvec(3);
      transstore(6*4+3,t)=0.5*scratchvec(3)*scratchvec(3);
    }
  }

  if (RE<C) {
    scratchvec(0)=1;//A->A  
    scratchvec(1)=0;//A->N
    scratchvec(3)= 
      (1 - (1-p))/
      (1 - (1-p) * exp( -tau * x[RE]) );//N->N
    scratchvec(2)=1-scratchvec(3); //N->A
    
    transstore(0*7+0,RE)=    scratchvec(0)*scratchvec(0);
    transstore(0*7+1,RE)=0;//scratchvec(0)*scratchvec(1);
    transstore(0*7+2,RE)=0;//scratchvec(1)*scratchvec(0);
    transstore(0*7+3,RE)=0;//scratchvec(1)*scratchvec(1);
    transstore(0*7+4,RE)=0;//scratchvec(0)*scratchvec(1);
    transstore(0*7+5,RE)=0;//scratchvec(1)*scratchvec(0);
    transstore(0*7+6,RE)=0;//scratchvec(1)*scratchvec(1);
    
    transstore(1*7+0,RE)=    scratchvec(0)*scratchvec(2);
    transstore(1*7+1,RE)=0.5*scratchvec(0)*scratchvec(3);
    transstore(1*7+2,RE)=0;//0.5*scratchvec(1)*scratchvec(2);
    transstore(1*7+3,RE)=0;//0.5*scratchvec(1)*scratchvec(3);
    transstore(1*7+4,RE)=0.5*scratchvec(0)*scratchvec(3);//special, because statespace at C
    transstore(1*7+5,RE)=0;//0.5*scratchvec(0)*scratchvec(3);
    transstore(1*7+6,RE)=0;//0.5*scratchvec(1)*scratchvec(3);
    
    transstore(2*7+0,RE)=    scratchvec(2)*scratchvec(0);
    transstore(2*7+1,RE)=0;//0.5*scratchvec(2)*scratchvec(1);
    transstore(2*7+2,RE)=0.5*scratchvec(3)*scratchvec(0);
    transstore(2*7+3,RE)=0;//0.5*scratchvec(3)*scratchvec(1);
    transstore(2*7+4,RE)=0;//0.5*scratchvec(3)*scratchvec(0);
    transstore(2*7+5,RE)=0.5*scratchvec(3)*scratchvec(0);
    transstore(2*7+6,RE)=0;//0.5*scratchvec(3)*scratchvec(1);
    
    transstore(3*7+0,RE)=    scratchvec(2)*scratchvec(2);
    transstore(3*7+1,RE)=0.5*scratchvec(2)*scratchvec(3);
    transstore(3*7+2,RE)=0.5*scratchvec(3)*scratchvec(2);
    transstore(3*7+3,RE)=0.5*scratchvec(3)*scratchvec(3);
    transstore(3*7+4,RE)=0.5*scratchvec(2)*scratchvec(3);
    transstore(3*7+5,RE)=0.5*scratchvec(3)*scratchvec(2);
    transstore(3*7+6,RE)=0.5*scratchvec(3)*scratchvec(3);
  }
 
  if (debug) {
    cout << "transstore" << endl;
    for (i=0;i<transstore.dimx();i++) {
      for (j=0;j<transstore.dimy();j++) {
	cout << transstore(i,j) << " "; }
      cout << endl;
    }
  }
   
}

void maketransCgenmkv2(dgrid &transstoreC, dvector &scratchvec, 
		       const dvector &y, 
		       const double &tau, const double &p,
		       const int &C, const int &LE, const int &RE,
		       const int &debug)
{
  int i,j,t;
  int LHSle=0,RHSre=0;

  if (LE<C && C<RE) {
    if (LE<C-1) LHSle=C-2; else LHSle=C-1;
    if (RE>C+1) RHSre=C+2; else RHSre=C+1;

    for (t=LHSle;t<C;t++) {
      scratchvec(0)= exp( -tau * y[t]);//A<-A  
      scratchvec(1)=0;//A<-N 
      scratchvec(2)= 1- scratchvec(0);//N<-A
      scratchvec(3)=1;//N<-N
      
      if (t==C-1) {
	transstoreC(0*7+0,t-(C-2))=    scratchvec(0)*scratchvec(0);
	transstoreC(0*7+1,t-(C-2))=0;//scratchvec(0)*scratchvec(1);
	transstoreC(0*7+2,t-(C-2))=0;//scratchvec(1)*scratchvec(0);
	transstoreC(0*7+3,t-(C-2))=0;//scratchvec(1)*scratchvec(1);
	transstoreC(0*7+4,t-(C-2))=0;//scratchvec(0)*scratchvec(1);
	transstoreC(0*7+5,t-(C-2))=0;//scratchvec(1)*scratchvec(0);
	transstoreC(0*7+6,t-(C-2))=0;//scratchvec(1)*scratchvec(1);
	
	transstoreC(1*7+0,t-(C-2))=0.5*scratchvec(0)*scratchvec(2);
	transstoreC(1*7+1,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
	transstoreC(1*7+2,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(2);
	transstoreC(1*7+3,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(3);
	transstoreC(1*7+4,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
	transstoreC(1*7+5,t-(C-2))=0;//0.5*scratchvec(0)*scratchvec(3);
	transstoreC(1*7+6,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(3);
	
	transstoreC(2*7+0,t-(C-2))=0.5*scratchvec(2)*scratchvec(0);
	transstoreC(2*7+1,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(1);
	transstoreC(2*7+2,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
	transstoreC(2*7+3,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(1);
	transstoreC(2*7+4,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(0);
	transstoreC(2*7+5,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
	transstoreC(2*7+6,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(1);
	
	transstoreC(3*7+0,t-(C-2))=0.5*scratchvec(2)*scratchvec(2);
	transstoreC(3*7+1,t-(C-2))=0.5*scratchvec(2)*scratchvec(3);
	transstoreC(3*7+2,t-(C-2))=0.5*scratchvec(3)*scratchvec(2);
	transstoreC(3*7+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(3);
	transstoreC(3*7+4,t-(C-2))=0.5*scratchvec(2)*scratchvec(3);
	transstoreC(3*7+5,t-(C-2))=0.5*scratchvec(3)*scratchvec(2);
	transstoreC(3*7+6,t-(C-2))=0.5*scratchvec(3)*scratchvec(3); }
      else {
	transstoreC(0*7+0,t-(C-2))=    scratchvec(0)*scratchvec(0);
	transstoreC(0*7+1,t-(C-2))=0;//scratchvec(0)*scratchvec(1);
	transstoreC(0*7+2,t-(C-2))=0;//scratchvec(1)*scratchvec(0);
	transstoreC(0*7+3,t-(C-2))=0;//scratchvec(1)*scratchvec(1);
	transstoreC(0*7+4,t-(C-2))=0;//scratchvec(0)*scratchvec(1);
	transstoreC(0*7+5,t-(C-2))=0;//scratchvec(1)*scratchvec(0);
	transstoreC(0*7+6,t-(C-2))=0;//scratchvec(1)*scratchvec(1);
	
	transstoreC(1*7+0,t-(C-2))=0.5*scratchvec(0)*scratchvec(2);
	transstoreC(1*7+1,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
	transstoreC(1*7+2,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(2);
	transstoreC(1*7+3,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(3);
	transstoreC(1*7+4,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(2);
	transstoreC(1*7+5,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
	transstoreC(1*7+6,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(3);
	
	transstoreC(2*7+0,t-(C-2))=0.5*scratchvec(2)*scratchvec(0);
	transstoreC(2*7+1,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(1);
	transstoreC(2*7+2,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
	transstoreC(2*7+3,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(1);
	transstoreC(2*7+4,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
	transstoreC(2*7+5,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(1);
	transstoreC(2*7+6,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(1);
	
	transstoreC(3*7+0,t-(C-2))=0.5*scratchvec(2)*scratchvec(2);
	transstoreC(3*7+1,t-(C-2))=0.5*scratchvec(2)*scratchvec(3);
	transstoreC(3*7+2,t-(C-2))=0.5*scratchvec(3)*scratchvec(2);
	transstoreC(3*7+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(3);
	transstoreC(3*7+4,t-(C-2))=0.5*scratchvec(3)*scratchvec(2);
	transstoreC(3*7+5,t-(C-2))=0.5*scratchvec(2)*scratchvec(3);
	transstoreC(3*7+6,t-(C-2))=0.5*scratchvec(3)*scratchvec(3); }


    }

    for (t=C;t<RHSre;t++) {
      scratchvec(0)= exp( -tau * y[t]);//A->A  
      scratchvec(1)=1- scratchvec(0);//A->N 
      scratchvec(2)=0;//N->A
      scratchvec(3)=1;//N->N

      transstoreC(0*4+0,t-(C-2))=    scratchvec(0)*scratchvec(0);
      transstoreC(0*4+1,t-(C-2))=0.5*scratchvec(0)*scratchvec(1);
      transstoreC(0*4+2,t-(C-2))=0.5*scratchvec(1)*scratchvec(0);
      transstoreC(0*4+3,t-(C-2))=0.5*scratchvec(1)*scratchvec(1);
     
      transstoreC(1*4+0,t-(C-2))=0;//scratchvec(0)*scratchvec(2);
      transstoreC(1*4+1,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
      transstoreC(1*4+2,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(2);
      transstoreC(1*4+3,t-(C-2))=0.5*scratchvec(1)*scratchvec(3);

      transstoreC(2*4+0,t-(C-2))=0;//scratchvec(2)*scratchvec(0);
      transstoreC(2*4+1,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(1);
      transstoreC(2*4+2,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
      transstoreC(2*4+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(1);

      transstoreC(3*4+0,t-(C-2))=0;//scratchvec(2)*scratchvec(2);
      transstoreC(3*4+1,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(3);
      transstoreC(3*4+2,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(2);
      transstoreC(3*4+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(3);

      transstoreC(4*4+0,t-(C-2))=0;//scratchvec(0)*scratchvec(2);
      transstoreC(4*4+1,t-(C-2))=0;//0.5*scratchvec(1)*scratchvec(2);
      transstoreC(4*4+2,t-(C-2))=0.5*scratchvec(0)*scratchvec(3);
      transstoreC(4*4+3,t-(C-2))=0.5*scratchvec(1)*scratchvec(3);

      transstoreC(5*4+0,t-(C-2))=0;//scratchvec(2)*scratchvec(0);
      transstoreC(5*4+1,t-(C-2))=0.5*scratchvec(3)*scratchvec(0);
      transstoreC(5*4+2,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(1);
      transstoreC(5*4+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(1);

      transstoreC(6*4+0,t-(C-2))=0;//scratchvec(2)*scratchvec(2);
      transstoreC(6*4+1,t-(C-2))=0;//0.5*scratchvec(3)*scratchvec(2);
      transstoreC(6*4+2,t-(C-2))=0;//0.5*scratchvec(2)*scratchvec(3);
      transstoreC(6*4+3,t-(C-2))=0.5*scratchvec(3)*scratchvec(3);
    }

    if (debug) {
      cout << "transstoreC" << endl;
      for (i=0;i<transstoreC.dimx();i++) {
	for (j=0;j<transstoreC.dimy();j++) {
	  cout << transstoreC(i,j) << " "; }
	cout << endl; }
    }

  }
}



void makenullCdengenmkv2(dvectors &nullCden, const igrid &data, 
			 const igrid &freqallele, const dgrid &freqfreq,
			 const dgrid &jointfreq2, 
			 const ivector &allelecounts,
			 const int &LE, const int &RE, const int &C,
			 const int &debug)
{
  int i,g;
  const int numgen=data.dimx()/2;
  int indk,indi;
  double np0,np1,np0m1,np1m1;

  if (LE<C && C<RE) {

    for (g=0;g<numgen;g++) {

      //make nullCden[0]

      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+1), C+1);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+1), C+1);

      nullCden[0][3*g  ]=np1;
      nullCden[0][3*g+1]=np0;
      nullCden[0][3*g+2]=np0*np1;

      if (RE>C+1) {
	//make nullCden[1]
	for (indk=1;indk>=0;indk--) {
	  
	  np0m1=getcondfreq2lSMART(freqallele,freqfreq,allelecounts,
				   jointfreq2, C+1,
				   data(2*g+(1-indk), C+1),
				   data(2*g         , C+2));
	  np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+2), C+2);
	  np1m1=getcondfreq2lSMART(freqallele,freqfreq,allelecounts,
				   jointfreq2, C+1,
				   data(2*g+(  indk), C+1),
				   data(2*g+1       , C+2));
	  np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+2), C+2);
	  
	  nullCden[1][10*g+7*(1-indk)  ]=          np1m1;
	  nullCden[1][10*g+7*(1-indk)+1]=      np0*np1m1;
	  nullCden[1][10*g+3*(1-indk)+2]=np0m1;
	  nullCden[1][10*g+3*(1-indk)+3]=np0m1*          np1;
	  nullCden[1][10*g+5*(1-indk)+4]=np0m1*    np1m1;

	}
      }

      //make nullCden[2]

      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C-1), C-1);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C-1), C-1);

      nullCden[2][3*g  ]=np1;
      nullCden[2][3*g+1]=np0;
      nullCden[2][3*g+2]=np0*np1;

      if (LE<C-1) {
	//make nullCden[3]
	for (indi=1;indi>=0;indi--) {
	  
	  np0m1=getcondfreq2rSMART(freqallele,freqfreq,allelecounts,
				   jointfreq2, C-2,
				   data(2*g         , C-2),
				   data(2*g+(1-indi), C-1));
	  np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C-2), C-2);
	  np1m1=getcondfreq2rSMART(freqallele,freqfreq,allelecounts,
				   jointfreq2, C-2,
				   data(2*g+1       , C-2),
				   data(2*g+(  indi), C-1));
	  np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C-2), C-2);
	  
	  nullCden[3][10*g+7*(1-indi)  ]=          np1m1;
	  nullCden[3][10*g+7*(1-indi)+1]=      np0*np1m1;
	  nullCden[3][10*g+3*(1-indi)+2]=np0m1;
	  nullCden[3][10*g+3*(1-indi)+3]=np0m1*          np1;
	  nullCden[3][10*g+5*(1-indi)+4]=np0m1*    np1m1;

	}
      }

    }
    if (debug) {
      cout << "nullCden" << endl;
      for (i=3*0;i<3*(0+1);i++)   cout << nullCden[0][i] << " "; cout << endl;
      for (i=10*0;i<10*(0+1);i++) cout << nullCden[1][i] << " "; cout << endl;
      for (i=3*0;i<3*(0+1);i++)   cout << nullCden[2][i] << " "; cout << endl;
      for (i=10*0;i<10*(0+1);i++) cout << nullCden[3][i] << " "; cout << endl; 
    }
  }
}

void makeobsCdengenmkv2(dvectors &obsCden, const dvectors &nullCden,
			const igrid &data,
			const dvector &mutmatch, dvector &mutnomatch,
			const ivector &anchap, 
			const int &C, const int &LE, const int &RE,
			const int &debug)
{
  int g,i;
  const int numgen=data.dimx()/2;
  
  double val=0;
  //reset
  obsCden[0]=nullCden[0];
  if (RE>C+1) obsCden[1]=nullCden[1];
  obsCden[2]=nullCden[2];
  if (LE<C-1) obsCden[3]=nullCden[3];

  for (g=0;g<numgen;g++) {
    //update obsCden[0]
    if (data(2*g,C+1)!=0) {
      if (data(2*g,C+1)==anchap(C+1)) val=mutmatch(C+1);
      else val=mutnomatch(C+1);
      obsCden[0][3*g  ]*=val; } 
    
    if (data(2*g+1,C+1)!=0) {
      if (data(2*g+1,C+1)==anchap(C+1)) val=mutmatch(C+1);	
      else val=mutnomatch(C+1);
      obsCden[0][3*g+1]*=val; } 
    
    if (RE>C+1) {
      //update obsCden[1]
      if (data(2*g,C+2)!=0) {
	if (data(2*g,C+2)==anchap(C+2)) val=mutmatch(C+2);
	else val=mutnomatch(C+2);
	obsCden[1][10*g  ]*=val; 
	obsCden[1][10*g+7]*=val; } 
      
      if (data(2*g+1,C+2)!=0) {
	if (data(2*g+1,C+2)==anchap(C+2)) val=mutmatch(C+2);
	else val=mutnomatch(C+2);
	obsCden[1][10*g+2]*=val;
	obsCden[1][10*g+5]*=val; } }

    //update obsCden[2]
    if (data(2*g,C-1)!=0) {
      if (data(2*g,C-1)==anchap(C-1)) val=mutmatch(C-1);
      else val=mutnomatch(C-1);
      obsCden[2][3*g  ]*=val; }

    if (data(2*g+1,C-1)!=0) {
      if (data(2*g+1,C-1)==anchap(C-1)) val=mutmatch(C-1);
      else val=mutnomatch(C-1);
      obsCden[2][3*g+1]*=val; }

    if (LE<C-1) {
      //update obsCden[3]
      if (data(2*g,C-2)!=0) {
	if (data(2*g,C-2)==anchap(C-2)) val=mutmatch(C-2);
	else val=mutnomatch(C-2);
	  obsCden[3][10*g  ]*=val; 
	  obsCden[3][10*g+7]*=val; } 
      
      if (data(2*g+1,C-2)!=0) {
	if (data(2*g+1,C-2)==anchap(C-2)) val=mutmatch(C-2);
	else val=mutnomatch(C-2);
	obsCden[3][10*g+2]*=val; 
	obsCden[3][10*g+5]*=val; } } 

  }
  if (debug) {
    cout << "obsCden" << endl;
    for (i=3*1;i<3*(1+1);i++)   cout << obsCden[0][i] << " "; cout << endl;
    for (i=10*1;i<10*(1+1);i++) cout << obsCden[1][i] << " "; cout << endl;
    for (i=3*1;i<3*(1+1);i++)   cout << obsCden[2][i] << " "; cout << endl;
    for (i=10*1;i<10*(1+1);i++) cout << obsCden[3][i] << " "; cout << endl; 
  }
}


void makeloglikgenmkv2(dvector &loglikvec, const dgrid &alpha, 
		       const dgrid &initstore, const dgrid &obsprobs,
		       const int &C, const int &RE)
{
  //makes vector of loglikelihoods for each genotype from alpha and
  //unconditional distribution of chain at one locus

  int i,g,tsi;
  double lik;
  const int numgen=alpha.dimx()/7;
  
  for (g=0;g<numgen;g++) {
    loglikvec[g]=0;
    lik=0;
    for (i=0;i<7;i++) {
      if (i<4) tsi=0; else tsi=3;
      if (C<RE) lik+=alpha(7*g+i,RE)*initstore(i-tsi,2)
		  *obsprobs(13*g+i-tsi,RE);//assumes O_C=9
      else lik+=alpha(7*g+i,C)*initstore(i-tsi,1);
    }
    //delete me
    //if (g==5)
    //cout << "lik " << g << " " << lik << endl;
    assert(0<lik && lik<=1.0001);
    loglikvec[g]=log(lik);
  }
}

void makegammagenmkv2(dgrid &gamma, const dgrid &alpha, const dgrid &beta,
		      const int &C, const int &LE, const int &RE, 
		      const dvectors &gammaC)
{
  //makes gamma object, for genotype data for affecteds
  
  int i,j,k,g,t;
  double denom, sum;

  const int numgen=gamma.dimx()/7;

  for (g=0;g<numgen;g++) {

    //calculate likelihood for genotype
    denom=0;
    if ( !(LE<C && C<RE)) 
      for (i=0;i<7;i++) { denom+=alpha(7*g+i,C)*beta(7*g+i,C); }
    else {
      for (i=0;i<49;i++) { denom+=gammaC[0][49*g+i]*gammaC[1][49*g+i]; } }
    

    //delete me
    //if (g==9) { 
    //cout << "gamma denominators " << g << endl;
    //cout << "C " << denom << endl;
    //for (t=LE;t<=RE;t++) {
    //denom=0;
    //for (i=0;i<7;i++) { denom+=alpha(7*g+i,t)*beta(7*g+i,t); }
    //cout << denom << " ";
    //}
    //cout << endl;
    //exit(1);
    //}

    if (C<LE || RE<C) {
      for (i=0;i<7;i++) { 
	gamma(7*g+i,C)=( alpha(7*g+i,C)*beta(7*g+i,C) )/denom; } }

    for (t=LE;t<=RE;t++) {

      if (t==C) {
	for (j=0;j<7;j++) {
	  sum=0;
	  for (k=0;k<7;k++)
	    sum+=gammaC[0][49*g+7*j+k]*gammaC[1][49*g+7*j+k];
	  gamma(7*g+j,C)=sum/denom;
	}
      }
      else {
	for (i=0;i<7;i++) { 
	  gamma(7*g+i,t)=alpha(7*g+i,t)*beta(7*g+i,t)/denom; } }
	
      //check that gamma's sum to 1 at each locus
      sum=0;
      for (i=0;i<7;i++) sum+=gamma(7*g+i,t);
      
      if ( !(0.999<sum && sum<1.001) ) {
	errorfs << "ERROR: gamma's do not sum to 1.0 (" << sum <<") for gen. "  
		<< g << "+1 and locus " << t << "+1" << endl; 
	exit(1);
      
      }
      
    }
  }
  
  for (g=0;g<gamma.dimx();g++) {
    if (C<LE || RE<C) 
      if ( !(-0.00001<gamma(g,C) && gamma(g,C)<1.00001) ) {
	errorfs << "ERROR: gamma(" << g << "," << C << ")="
		<< gamma(g,C) << endl;
	exit(1);
	assert(-0.00001<gamma(g,C) && gamma(g,C)<1.00001); }
    for (t=LE;t<=RE;t++)
      if ( !(-0.00001<gamma(g,t) && gamma(g,t)<1.00001) ) {
	errorfs << "ERROR: gamma(" << g << "," << t << ")="
		<< gamma(g,t) << endl;
	exit(1);
	assert(-0.00001<gamma(g,t) && gamma(g,t)<1.00001); }
  }
  //delete me (remove comments above too)
  //exit(1);
}

void makecstargenmkv2(dvector &cstar, const dgrid &gamma, 
		      const int &C, const int &LE, const int &RE)
{
  //makes the cstar, expected counts of haplotypes still sharing from
  //ancestral haplotype at each marker, for genotype data

  int i,g,t;
  const int numgen=gamma.dimx()/7;

  if (C<LE || RE<C) {
    cstar(C)=0;
    for (g=0;g<numgen;g++) {
      cstar(C)+=2*gamma(7*g,C);
      for (i=0;i<2;i++) {
	cstar(C)+=gamma(7*g+1 +3*i,C)+gamma(7*g+2 +3*i,C); } } }

  for (t=LE;t<=RE;t++) {
    cstar(t)=0;
    for (g=0;g<numgen;g++) {
      cstar(t)+=2*gamma(7*g,t);
      for (i=0;i<2;i++) {
	cstar(t)+=gamma(7*g+1 +3*i,t)+gamma(7*g+2 +3*i,t); } } }
  
  //assertions
  for (t=LE;t<=RE;t++) {
    assert(cstar(t)<=cstar(C)); }

} 

void makecstarmutgenmkv2(dvector &cstarmut, const dvector &cstar,
			 const dgrid &gamma,
			 const igrid &data, const ivector &anchap,
			 const dvector &mutmatch,
			 const int &C, const int &LE, const int &RE)
{
  //makes subset of cstars which do not match ancestral hap., only
  //due to mutation, for genotype data

  int g,l,t,hap;
  const int numgen=gamma.dimx()/7;
  double sum;

  //assumes no mutation allowed at center
  cstarmut(C)=0;  

  for (t=LE;t<=RE;t++) {
    if (t!=C) {
      cstarmut(t)=0;
      for (g=0;g<numgen;g++) {
	
	for (hap=0;hap<=1;hap++) {
	  sum=0;
	  if (data(2*g+hap,t)!=anchap(t)) {
	    if (data(2*g+hap,t)==0) { 
	      sum=gamma(7*g  ,t);
	      for (l=0;l<2;l++) {
		sum+=gamma(7*g+ 3*l +1+hap  ,t); }
	      cstarmut(t)+=(1-mutmatch[t])*sum; }
	    else {
	      sum=gamma(7*g  ,t);
	      for (l=0;l<2;l++) {
		sum+=gamma(7*g+ 3*l +1+hap  ,t); }
	      cstarmut(t)+=sum; } }
	  
	}
	
      }
    }
  }
}

void makenullCnumgenmkv2(dvectors &nullCnum, const igrid &data,
			 const igrid &freqallele, const dgrid &freqfreq,
			 const dgrid &jointfreq2, 
			 const dgrid &jointfreq3mkv2,
                         const dgrid &jointfreq3fmkv2, 
			 const dgrid &jointfreq4mkv2,
			 const dgrid &jointfreq4fmkv2,
			 const ivector &allelecounts,
			 const int &LE, const int &RE, const int &C,
			 const int &debug)
{
  const int numgen=data.dimx()/2;

  int i,g,indi,indj,indk;
  double np0,np1,np0m1,np1m1,np0m2,np1m2;

  //reset
  for (i=0;i<nullCnum[0].dim();i++) nullCnum[0][i]=0.0;
  if (RE>C+1) for (i=0;i<nullCnum[1].dim();i++) nullCnum[1][i]=0.0;
  for (i=0;i<nullCnum[2].dim();i++) nullCnum[2][i]=0.0;
  if (LE<C-1) for (i=0;i<nullCnum[3].dim();i++) nullCnum[3][i]=0.0;

  for (g=0;g<numgen;g++) {

    //obsCnum[0]
    for (indi=1;indi>=0;indi--) {
      for (indj=1;indj>=0;indj--) {
	
	if (LE<C-1) {
	  np0m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g+  (indj-indi)*(indj-indi),C-2), 
				   data(2*g+(1-indj),C-1),
				   9, 
				   data(2*g  ,C+1),
				   C);
	  np1m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g+1-(indj-indi)*(indj-indi),C-2), 
				   data(2*g+(  indj),C-1),
				   9, 
				   data(2*g+1,C+1),
				   C); }
	else {
	  np0m2=getcondfreq3lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq3fmkv2, C-1, 
				   data(2*g+(1-indj),C-1),
				   9, 
				   data(2*g  ,C+1));
	  np1m2=getcondfreq3lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq3fmkv2, C-1, 
				   data(2*g+(  indj),C-1),
				   9, 
				   data(2*g+1,C+1)); }
	np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+1), C+1);
	np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+1), C+1);
	
	if (indj==1 && indi==1) nullCnum[0][20*g   ]=      np1m2;
	if (indj==1 && indi==1) nullCnum[0][20*g+ 1]=np0  *np1m2;
	if (indj==1 && indi==0) nullCnum[0][20*g+ 2]=      np1m2;
	if (indj==1 && indi==0) nullCnum[0][20*g+ 3]=np0  *np1m2;

	if (indj==1 && indi==1) nullCnum[0][20*g+ 4]=np0m2      ;
	if (indj==1 && indi==1) nullCnum[0][20*g+ 5]=np0m2*np1  ;
	if (indj==1 && indi==0) nullCnum[0][20*g+ 6]=np0m2      ;
	if (indj==1 && indi==0) nullCnum[0][20*g+ 7]=np0m2*np1  ;

	if (indj==1 && indi==1) nullCnum[0][20*g+ 8]=np0m2*np1m2;
	if (indj==1 && indi==0) nullCnum[0][20*g+ 9]=np0m2*np1m2;

	if (indj==0 && indi==1) nullCnum[0][20*g+10]=np0m2      ;
	if (indj==0 && indi==1) nullCnum[0][20*g+11]=np0m2*np1  ;
	if (indj==0 && indi==0) nullCnum[0][20*g+12]=np0m2      ;
	if (indj==0 && indi==0) nullCnum[0][20*g+13]=np0m2*np1  ;

	if (indj==0 && indi==1) nullCnum[0][20*g+14]=      np1m2;
	if (indj==0 && indi==1) nullCnum[0][20*g+15]=np0  *np1m2;
	if (indj==0 && indi==0) nullCnum[0][20*g+16]=      np1m2;
	if (indj==0 && indi==0) nullCnum[0][20*g+17]=np0  *np1m2;

	if (indj==0 && indi==1) nullCnum[0][20*g+18]=np0m2*np1m2;
	if (indj==0 && indi==0) nullCnum[0][20*g+19]=np0m2*np1m2;
      }
    }

    if (RE>C+1) {

      //obsCnum[1]
      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+2), C+2);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+2), C+2);
      for (indk=1;indk>=0;indk--) {
	for (indj=1;indj>=0;indj--) {

	  np0m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g+  (indj-indk)*(indj-indk),C-1), 
				   9,
				   data(2*g+(1-indk),C+1), 
				   data(2*g  ,C+2),
				   C);
	  np1m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g+1-(indj-indk)*(indj-indk),C-1), 
				   9,
				   data(2*g+(  indk),C+1), 
				   data(2*g+1,C+2),
				   C);

	  np0m1=getcondfreq2lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, C+1,
				   data(2*g+(1-indk),C+1), 
				   data(2*g  ,C+2));
	  np1m1=getcondfreq2lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, C+1,
				   data(2*g+(  indk),C+1), 
				   data(2*g+1,C+2));

	  if (indj==1 && indk==1) nullCnum[1][28*g   ]=      np1m2;
	  if (indj==1 && indk==1) nullCnum[1][28*g+ 1]=np0*  np1m2;
	  if (indj==1 && indk==1) nullCnum[1][28*g+ 2]=np0m1*np1m2;
	  if (indj==1 && indk==0) nullCnum[1][28*g+ 3]=np0m2;
	  if (indj==1 && indk==0) nullCnum[1][28*g+ 4]=np0m2*np1;
	  if (indj==1 && indk==0) nullCnum[1][28*g+ 5]=np0m2*np1m1; 

  	  if (indj==1 && indk==1) nullCnum[1][28*g+ 6]=np0m2;
	  if (indj==1 && indk==1) nullCnum[1][28*g+ 7]=np0m2*np1;
	  if (indj==1 && indk==1) nullCnum[1][28*g+ 8]=np0m2*np1m1; 
	  if (indj==1 && indk==0) nullCnum[1][28*g+ 9]=      np1m2;
	  if (indj==1 && indk==0) nullCnum[1][28*g+10]=np0*  np1m2;
	  if (indj==1 && indk==0) nullCnum[1][28*g+11]=np0m1*np1m2;

	  if (indj==1 && indk==1) nullCnum[1][28*g+12]=np0m2*np1m2; 
	  if (indj==1 && indk==0) nullCnum[1][28*g+13]=np0m2*np1m2; 

	  if (indj==0 && indk==1) nullCnum[1][28*g+14]=np0m2;
	  if (indj==0 && indk==1) nullCnum[1][28*g+15]=np0m2*np1;
	  if (indj==0 && indk==1) nullCnum[1][28*g+16]=np0m2*np1m1; 
	  if (indj==0 && indk==0) nullCnum[1][28*g+17]=      np1m2;
	  if (indj==0 && indk==0) nullCnum[1][28*g+18]=np0*  np1m2;
	  if (indj==0 && indk==0) nullCnum[1][28*g+19]=np0m1*np1m2;

	  if (indj==0 && indk==1) nullCnum[1][28*g+20]=      np1m2;
	  if (indj==0 && indk==1) nullCnum[1][28*g+21]=np0*  np1m2;
	  if (indj==0 && indk==1) nullCnum[1][28*g+22]=np0m1*np1m2;
	  if (indj==0 && indk==0) nullCnum[1][28*g+23]=np0m2;
	  if (indj==0 && indk==0) nullCnum[1][28*g+24]=np0m2*np1;
	  if (indj==0 && indk==0) nullCnum[1][28*g+25]=np0m2*np1m1; 

	  if (indj==0 && indk==1) nullCnum[1][28*g+26]=np0m2*np1m2; 
	  if (indj==0 && indk==0) nullCnum[1][28*g+27]=np0m2*np1m2; 
	}
      }
      
    }

    //obsCnum[2]
    np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C-1), C-1);
    np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C-1), C-1);
    for (indk=1;indk>=0;indk--) {
      for (indj=1;indj>=0;indj--) {

	if (RE>C+1) {
	  np0m2=getcondfreq4rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g  ,C-1),
				   9,
				   data(2*g+(1-indj),C+1),
				   data(2*g+  (indj-indk)*(indj-indk),C+2),
				   C);
	  np1m2=getcondfreq4rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g+1,C-1),
				   9,
				   data(2*g+(  indj),C+1),
				   data(2*g+1-(indj-indk)*(indj-indk),C+2),
				   C); }
	else {
	  np0m2=getcondfreq3rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq3fmkv2, C-1, 
				   data(2*g         ,C-1),
				   9, 
				   data(2*g+(1-indj),C+1));
	  np1m2=getcondfreq3rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq3fmkv2, C-1, 
				   data(2*g+1       ,C-1),
				   9, 
				   data(2*g+(  indj),C+1)); }

	if (indj==1 && indk==1) nullCnum[2][23*g   ]=      np1  ;
	if (indj==1 && indk==1) nullCnum[2][23*g+ 1]=      np1m2;
	if (indj==1 && indk==0) nullCnum[2][23*g+ 2]=      np1m2;
	if (indj==0 && indk==1) nullCnum[2][23*g+ 3]=      np1m2;
	if (indj==0 && indk==0) nullCnum[2][23*g+ 4]=      np1m2;

	if (indj==1 && indk==1) nullCnum[2][23*g+ 5]=np0        ;
	if (indj==1 && indk==1) nullCnum[2][23*g+ 6]=np0m2      ;
	if (indj==1 && indk==0) nullCnum[2][23*g+ 7]=np0m2      ;
	if (indj==0 && indk==1) nullCnum[2][23*g+ 8]=np0m2      ;
	if (indj==0 && indk==0) nullCnum[2][23*g+ 9]=np0m2      ;

	if (indj==1 && indk==1) nullCnum[2][23*g+10]=np0  *np1  ;

	if (indj==1 && indk==1) nullCnum[2][23*g+11]=np0  *np1m2;
	if (indj==1 && indk==0) nullCnum[2][23*g+12]=np0  *np1m2;
	if (indj==1 && indk==1) nullCnum[2][23*g+13]=np0m2*np1  ;
	if (indj==1 && indk==0) nullCnum[2][23*g+14]=np0m2*np1  ;
	if (indj==1 && indk==1) nullCnum[2][23*g+15]=np0m2*np1m2;
	if (indj==1 && indk==0) nullCnum[2][23*g+16]=np0m2*np1m2;
	if (indj==0 && indk==1) nullCnum[2][23*g+17]=np0  *np1m2;
	if (indj==0 && indk==0) nullCnum[2][23*g+18]=np0  *np1m2;
	if (indj==0 && indk==1) nullCnum[2][23*g+19]=np0m2*np1  ;
	if (indj==0 && indk==0) nullCnum[2][23*g+20]=np0m2*np1  ;
	if (indj==0 && indk==1) nullCnum[2][23*g+21]=np0m2*np1m2;
	if (indj==0 && indk==0) nullCnum[2][23*g+22]=np0m2*np1m2;
      }
    }

    if (LE<C-1) {
      //obsCnum[3]
      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C-2), C-2);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C-2), C-2);
      for (indi=1;indi>=0;indi--) {

	np0m1=getcondfreq2rSMART(freqallele, freqfreq, allelecounts, 
				 jointfreq2, C-2,
				 data(2*g         ,C-2), 
				 data(2*g+(1-indi),C-1));
	np1m1=getcondfreq2rSMART(freqallele, freqfreq, allelecounts, 
				 jointfreq2, C-2,
				 data(2*g+1       ,C-2), 
				 data(2*g+(  indi),C-1));

	for (indj=1;indj>=0;indj--) {
	  
	  np0m2=getcondfreq4rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g  ,C-2),
				   data(2*g+(1-indi),C-1),
				   9,
				   data(2*g+  (indj-indi)*(indj-indi),C+1),
				   C); 
	  np1m2=getcondfreq4rSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g+1,C-2),
				   data(2*g+(  indi),C-1),
				   9,
				   data(2*g+1-(indj-indi)*(indj-indi),C+1),
				   C); 
	
	  
	  if (indi==1 && indj==1) nullCnum[3][38*g   ]=      np1m1;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 1]=np0  *np1m1;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 2]=      np1m2;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 3]=np0  *np1m2;
	  if (indi==1 && indj==0) nullCnum[3][38*g+ 4]=      np1m2;
	  if (indi==1 && indj==0) nullCnum[3][38*g+ 5]=np0  *np1m2;

	  if (indi==1 && indj==1) nullCnum[3][38*g+ 6]=np0m1      ;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 7]=np0m1*np1  ;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 8]=np0m2      ;
	  if (indi==1 && indj==1) nullCnum[3][38*g+ 9]=np0m2*np1  ;
	  if (indi==1 && indj==0) nullCnum[3][38*g+10]=np0m2      ;
	  if (indi==1 && indj==0) nullCnum[3][38*g+11]=np0m2*np1  ;
	  
	  if (indi==1 && indj==1) nullCnum[3][38*g+12]=np0m1*np1m1;
	  if (indi==1 && indj==1) nullCnum[3][38*g+13]=np0m1*np1m2;
	  if (indi==1 && indj==1) nullCnum[3][38*g+14]=np0m2*np1m1;
	  if (indi==1 && indj==1) nullCnum[3][38*g+15]=np0m2*np1m2;
	  if (indi==1 && indj==0) nullCnum[3][38*g+16]=np0m1*np1m2;
	  if (indi==1 && indj==0) nullCnum[3][38*g+17]=np0m2*np1m1;
	  if (indi==1 && indj==0) nullCnum[3][38*g+18]=np0m2*np1m2;

	  if (indi==0 && indj==1) nullCnum[3][38*g+19]=np0m1      ;
	  if (indi==0 && indj==1) nullCnum[3][38*g+20]=np0m1*np1  ;
	  if (indi==0 && indj==1) nullCnum[3][38*g+21]=np0m2      ;
	  if (indi==0 && indj==1) nullCnum[3][38*g+22]=np0m2*np1  ;
	  if (indi==0 && indj==0) nullCnum[3][38*g+23]=np0m2      ;
	  if (indi==0 && indj==0) nullCnum[3][38*g+24]=np0m2*np1  ;

	  if (indi==0 && indj==1) nullCnum[3][38*g+25]=      np1m1;
	  if (indi==0 && indj==1) nullCnum[3][38*g+26]=np0  *np1m1;
	  if (indi==0 && indj==1) nullCnum[3][38*g+27]=      np1m2;
	  if (indi==0 && indj==1) nullCnum[3][38*g+28]=np0  *np1m2;
	  if (indi==0 && indj==0) nullCnum[3][38*g+29]=      np1m2;
	  if (indi==0 && indj==0) nullCnum[3][38*g+30]=np0  *np1m2;

	  if (indi==0 && indj==1) nullCnum[3][38*g+31]=np0m1*np1m1;
	  if (indi==0 && indj==1) nullCnum[3][38*g+32]=np0m2*np1m1;
	  if (indi==0 && indj==1) nullCnum[3][38*g+33]=np0m1*np1m2;
	  if (indi==0 && indj==1) nullCnum[3][38*g+34]=np0m2*np1m2;
	  if (indi==0 && indj==0) nullCnum[3][38*g+35]=np0m2*np1m1;
	  if (indi==0 && indj==0) nullCnum[3][38*g+36]=np0m1*np1m2;
	  if (indi==0 && indj==0) nullCnum[3][38*g+37]=np0m2*np1m2;
	}
      }
    }

    //for alpha(,1)
    //obsCnum[4]
    np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+1), C+1);
    np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+1), C+1);
    for (indi=1;indi>=0;indi--) {
      for (indj=1;indj>=0;indj--) {

	if (LE<C-1) {
	  np0m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g+  (indj-indi)*(indj-indi) ,C-2),
				   data(2*g+(1-indj),C-1),
				   9,
				   data(2*g,C+1),
				   C);
	  np1m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-2, 
				   data(2*g+1-(indj-indi)*(indj-indi),C-2),
				   data(2*g+(  indj),C-1),
				   9,
				   data(2*g+1,C+1),
				   C); }
	else {
	  np0m2=getcondfreq3lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, 
				   jointfreq3fmkv2, C-1, 
				   data(2*g+(1-indj),C-1),
				   9, 
				   data(2*g,C+1));
	  np1m2=getcondfreq3lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, 
				   jointfreq3fmkv2, C-1, 
				   data(2*g+(  indj),C-1),
				   9, 
				   data(2*g+1,C+1)); }

	if (indj==1 && indi==1) nullCnum[4][23*g   ]=      np1  ;
	if (indj==1 && indi==1) nullCnum[4][23*g+ 1]=      np1m2;
	if (indj==1 && indi==0) nullCnum[4][23*g+ 2]=      np1m2;
	if (indj==0 && indi==1) nullCnum[4][23*g+ 3]=      np1m2;
	if (indj==0 && indi==0) nullCnum[4][23*g+ 4]=      np1m2;

	if (indj==1 && indi==1) nullCnum[4][23*g+ 5]=np0        ;
	if (indj==1 && indi==1) nullCnum[4][23*g+ 6]=np0m2      ;
	if (indj==1 && indi==0) nullCnum[4][23*g+ 7]=np0m2      ;
	if (indj==0 && indi==1) nullCnum[4][23*g+ 8]=np0m2      ;
	if (indj==0 && indi==0) nullCnum[4][23*g+ 9]=np0m2      ;

	if (indj==1 && indi==1) nullCnum[4][23*g+10]=np0  *np1  ;

	if (indj==1 && indi==1) nullCnum[4][23*g+11]=np0  *np1m2;
	if (indj==1 && indi==0) nullCnum[4][23*g+12]=np0  *np1m2;
	if (indj==1 && indi==1) nullCnum[4][23*g+13]=np0m2*np1  ;
	if (indj==1 && indi==0) nullCnum[4][23*g+14]=np0m2*np1  ;
	if (indj==1 && indi==1) nullCnum[4][23*g+15]=np0m2*np1m2;
	if (indj==1 && indi==0) nullCnum[4][23*g+16]=np0m2*np1m2;
	if (indj==0 && indi==1) nullCnum[4][23*g+17]=np0m2*np1  ;
	if (indj==0 && indi==0) nullCnum[4][23*g+18]=np0m2*np1  ;
	if (indj==0 && indi==1) nullCnum[4][23*g+19]=np0  *np1m2;
	if (indj==0 && indi==0) nullCnum[4][23*g+20]=np0  *np1m2;
	if (indj==0 && indi==1) nullCnum[4][23*g+21]=np0m2*np1m2;
	if (indj==0 && indi==0) nullCnum[4][23*g+22]=np0m2*np1m2;
      }
    }

    if (RE>C+1) {
      //obsCnum[5]
      np0=getfreqSMART(freqallele, freqfreq, data(2*g  , C+2), C+2);
      np1=getfreqSMART(freqallele, freqfreq, data(2*g+1, C+2), C+2);
      for (indk=1;indk>=0;indk--) {

	np0m1=getcondfreq2lSMART(freqallele, freqfreq, allelecounts, 
				 jointfreq2, C+1,
				 data(2*g+(1-indk),C+1 ),
				 data(2*g         ,C+2));
	np1m1=getcondfreq2lSMART(freqallele, freqfreq, allelecounts, 
				 jointfreq2, C+1,
				 data(2*g+(  indk),C+1),
				 data(2*g+1       ,C+2));

	for (indj=1;indj>=0;indj--) {
	  
	  np0m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g+  (indj-indk)*(indj-indk),C-1),
				   9,
				   data(2*g+(1-indk),C+1),
				   data(2*g  ,C+2),
				   C); 
	  np1m2=getcondfreq4lSMART(freqallele, freqfreq, allelecounts, 
				   jointfreq2, jointfreq3mkv2, jointfreq4mkv2,
				   jointfreq4fmkv2, C-1, 
				   data(2*g+1-(indj-indk)*(indj-indk),C-1),
				   9,
				   data(2*g+(  indk),C+1),
				   data(2*g+1,C+2),
				   C); 
	
	  
	  if (indk==1 && indj==1) nullCnum[5][38*g   ]=      np1m1;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 1]=np0  *np1m1;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 2]=      np1m2;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 3]=np0  *np1m2;
	  if (indk==1 && indj==0) nullCnum[5][38*g+ 4]=      np1m2;
	  if (indk==1 && indj==0) nullCnum[5][38*g+ 5]=np0  *np1m2;

	  if (indk==1 && indj==1) nullCnum[5][38*g+ 6]=np0m1      ;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 7]=np0m1*np1  ;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 8]=np0m2      ;
	  if (indk==1 && indj==1) nullCnum[5][38*g+ 9]=np0m2*np1  ;
	  if (indk==1 && indj==0) nullCnum[5][38*g+10]=np0m2      ;
	  if (indk==1 && indj==0) nullCnum[5][38*g+11]=np0m2*np1  ;
	  
	  if (indk==1 && indj==1) nullCnum[5][38*g+12]=np0m1*np1m1;
	  if (indk==1 && indj==1) nullCnum[5][38*g+13]=np0m1*np1m2;
	  if (indk==1 && indj==1) nullCnum[5][38*g+14]=np0m2*np1m1;
	  if (indk==1 && indj==1) nullCnum[5][38*g+15]=np0m2*np1m2;
	  if (indk==1 && indj==0) nullCnum[5][38*g+16]=np0m2*np1m1;
	  if (indk==1 && indj==0) nullCnum[5][38*g+17]=np0m1*np1m2;
	  if (indk==1 && indj==0) nullCnum[5][38*g+18]=np0m2*np1m2;

	  if (indk==0 && indj==1) nullCnum[5][38*g+19]=np0m1      ;
	  if (indk==0 && indj==1) nullCnum[5][38*g+20]=np0m1*np1  ;
	  if (indk==0 && indj==1) nullCnum[5][38*g+21]=np0m2      ;
	  if (indk==0 && indj==1) nullCnum[5][38*g+22]=np0m2*np1  ;
	  if (indk==0 && indj==0) nullCnum[5][38*g+23]=np0m2      ;
	  if (indk==0 && indj==0) nullCnum[5][38*g+24]=np0m2*np1  ;

	  if (indk==0 && indj==1) nullCnum[5][38*g+25]=      np1m1;
	  if (indk==0 && indj==1) nullCnum[5][38*g+26]=np0  *np1m1;
	  if (indk==0 && indj==1) nullCnum[5][38*g+27]=      np1m2;
	  if (indk==0 && indj==1) nullCnum[5][38*g+28]=np0  *np1m2;
	  if (indk==0 && indj==0) nullCnum[5][38*g+29]=      np1m2;
	  if (indk==0 && indj==0) nullCnum[5][38*g+30]=np0  *np1m2;

	  if (indk==0 && indj==1) nullCnum[5][38*g+31]=np0m1*np1m1;
	  if (indk==0 && indj==1) nullCnum[5][38*g+32]=np0m2*np1m1;
	  if (indk==0 && indj==1) nullCnum[5][38*g+33]=np0m1*np1m2;
	  if (indk==0 && indj==1) nullCnum[5][38*g+34]=np0m2*np1m2;
	  if (indk==0 && indj==0) nullCnum[5][38*g+35]=np0m1*np1m2;
	  if (indk==0 && indj==0) nullCnum[5][38*g+36]=np0m2*np1m1;
	  if (indk==0 && indj==0) nullCnum[5][38*g+37]=np0m2*np1m2;
	}
      }
    }
  }
  if (debug) {
    cout << "nullCnum" << endl;
    for (i=20*0;i<20*(0+1);i++) cout <<nullCnum[0][i] << " "; cout << endl;
    for (i=28*0;i<28*(0+1);i++) cout <<nullCnum[1][i] << " "; cout << endl;
    for (i=23*0;i<23*(0+1);i++) cout <<nullCnum[2][i] << " "; cout << endl;
    for (i=38*0;i<38*(0+1);i++) cout <<nullCnum[3][i] << " "; cout << endl; 
    for (i=23*0;i<23*(0+1);i++) cout <<nullCnum[4][i] << " "; cout << endl;
    for (i=38*0;i<38*(0+1);i++) cout <<nullCnum[5][i] << " "; cout << endl; 
    }
}

void makeobsCnumgenmkv2(dvectors &obsCnum, const dvectors &nullCnum,
			const igrid &data,
			const dvector &mutmatch, dvector &mutnomatch,
			const ivector &anchap, 
			const int &C, const int &LE, const int &RE,
			const int &debug)
{
  int g,i;
  const int numgen=data.dimx()/2;

  double val0=0,val1=0;
  if (LE<C && C<RE) {

    obsCnum[0]=nullCnum[0];
    if (C+1<RE) obsCnum[1]=nullCnum[1];
    obsCnum[2]=nullCnum[2];
    if (C-1>LE) obsCnum[3]=nullCnum[3];
    obsCnum[4]=nullCnum[4];
    if (C+1<RE) obsCnum[5]=nullCnum[5];

    for (g=0;g<numgen;g++) {
      //obsCnum[0]
      if (data(2*g  ,C+1) != 0) {
	if (data(2*g  ,C+1) == anchap(C+1)) val0=mutmatch(C+1);
	else val0=mutnomatch(C+1);
	
	obsCnum[0][20*g   ]*=val0;
	obsCnum[0][20*g+ 2]*=val0;
	obsCnum[0][20*g+14]*=val0;
	obsCnum[0][20*g+16]*=val0;
	
      }
      
      if (data(2*g+1,C+1) != 0) {
	if (data(2*g+1,C+1) == anchap(C+1)) val1=mutmatch(C+1);
	else val1=mutnomatch(C+1);
	
	obsCnum[0][20*g+ 4]*=val1;
	obsCnum[0][20*g+ 6]*=val1;
	obsCnum[0][20*g+10]*=val1;
	obsCnum[0][20*g+12]*=val1;
	
      }
      
      //obsCnum[1]
      if (C+1<RE) {
	if (data(2*g  ,C+2) != 0) {
	  if (data(2*g  ,C+2) == anchap(C+2)) val0=mutmatch(C+2);
	  else val0=mutnomatch(C+2);
	  
	  obsCnum[1][28*g   ]*=val0;
	  obsCnum[1][28*g+ 9]*=val0;
	  obsCnum[1][28*g+17]*=val0;
	  obsCnum[1][28*g+20]*=val0;
	  
	}
	
	if (data(2*g+1,C+2) != 0) {
	  if (data(2*g+1,C+2) == anchap(C+2)) val1=mutmatch(C+2);
	  else val1=mutnomatch(C+2);
	  
	  obsCnum[1][28*g+ 3]*=val1;
	  obsCnum[1][28*g+ 6]*=val1;
	  obsCnum[1][28*g+14]*=val1;
	  obsCnum[1][28*g+23]*=val1;
	  
	}
	
      }
      
      //obsCnum[2]
      if (data(2*g  ,C-1) != 0) {
	if (data(2*g  ,C-1) == anchap(C-1)) val0=mutmatch(C-1);
	else val0=mutnomatch(C-1);
	
	for (i=0;i<5;i++) obsCnum[2][23*g+ i]*=val0;
      }
      
      if (data(2*g+1,C-1) != 0) {
	if (data(2*g+1,C-1) == anchap(C-1)) val1=mutmatch(C-1);
	else val1=mutnomatch(C-1);
	
	for (i=5;i<10;i++) obsCnum[2][23*g+ i]*=val1;
	
      }
      
      //obsCnum[3]
      if (C-1>LE) {
	if (data(2*g  ,C-2) != 0) {
	  if (data(2*g  ,C-2) == anchap(C-2)) val0=mutmatch(C-2);
	  else val0=mutnomatch(C-2);
	  
	  obsCnum[3][38*g   ]*=val0;
	  obsCnum[3][38*g+ 2]*=val0;
	  obsCnum[3][38*g+ 4]*=val0;
	  obsCnum[3][38*g+25]*=val0;
	  obsCnum[3][38*g+27]*=val0;
	  obsCnum[3][38*g+29]*=val0;
	  
	}
	
	if (data(2*g+1,C-2) != 0) {
	  if (data(2*g+1,C-2) == anchap(C-2)) val1=mutmatch(C-2);
	  else val1=mutnomatch(C-2);

	  obsCnum[3][38*g+ 6]*=val1;
	  obsCnum[3][38*g+ 8]*=val1;
	  obsCnum[3][38*g+10]*=val1;
	  obsCnum[3][38*g+19]*=val1;
	  obsCnum[3][38*g+21]*=val1;
	  obsCnum[3][38*g+23]*=val1;
	  
	}
	
      }
      //obsCnum[4]
      if (data(2*g  ,C+1) != 0) {
	if (data(2*g  ,C+1) == anchap(C+1)) val0=mutmatch(C+1);
	else val0=mutnomatch(C+1);
	
	for (i=0;i<5;i++) obsCnum[4][23*g+ i]*=val0;
      }
      
      if (data(2*g+1,C+1) != 0) {
	if (data(2*g+1,C+1) == anchap(C+1)) val1=mutmatch(C+1);
	else val1=mutnomatch(C+1);
	
	for (i=5;i<10;i++) obsCnum[4][23*g+ i]*=val1;
	
      }
      
      //obsCnum[5]
      if (C+1<RE) {
	if (data(2*g  ,C+2) != 0) {
	  if (data(2*g  ,C+2) == anchap(C+2)) val0=mutmatch(C+2);
	  else val0=mutnomatch(C+2);
	  
	  obsCnum[5][38*g   ]*=val0;
	  obsCnum[5][38*g+ 2]*=val0;
	  obsCnum[5][38*g+ 4]*=val0;
	  obsCnum[5][38*g+25]*=val0;
	  obsCnum[5][38*g+27]*=val0;
	  obsCnum[5][38*g+29]*=val0;
	  
	}
	
	if (data(2*g+1,C+2) != 0) {
	  if (data(2*g+1,C+2) == anchap(C+2)) val1=mutmatch(C+2);
	  else val1=mutnomatch(C+2);

	  obsCnum[5][38*g+ 6]*=val1;
	  obsCnum[5][38*g+ 8]*=val1;
	  obsCnum[5][38*g+10]*=val1;
	  obsCnum[5][38*g+19]*=val1;
	  obsCnum[5][38*g+21]*=val1;
	  obsCnum[5][38*g+23]*=val1;
	  
	}
	
      }

    }

    if (debug) {
      cout << "obsCnum" << endl;
      for (i=20*1;i<20*(1+1);i++) cout <<obsCnum[0][i] << " "; cout << endl;
      for (i=28*1;i<28*(1+1);i++) cout <<obsCnum[1][i] << " "; cout << endl;
      for (i=23*1;i<23*(1+1);i++) cout <<obsCnum[2][i] << " "; cout << endl;
      for (i=38*1;i<38*(1+1);i++) cout <<obsCnum[3][i] << " "; cout << endl; 
      for (i=23*1;i<23*(1+1);i++) cout <<obsCnum[4][i] << " "; cout << endl;
      for (i=38*1;i<38*(1+1);i++) cout <<obsCnum[5][i] << " "; cout << endl; 
    }
    
    
  }
}

void makealphagenmkv2(dgrid &alpha, const dgrid &initstore, 
		      const dgrid &transstore, const dgrid &transstoreC,
		      const dgrid &obsprobs, const dvectors &obsCnum,
		      const dvectors &obsCden, 
		      const int &C, const int &LE, const int &RE)
{

  //makes alpha object, when genotypes from affecteds are available

  int i,j,g,t,indi,indj,indk,tsi,tsj;
  const int numgen=alpha.dimx()/7;
  int LHSle, LHSre, RHSle, RHSre;
  double sum,prod2;

  //these are endpoints for t, rather than t+1
  if (LE<C && C<RE) {
    LHSle=LE; LHSre=C-2;
    RHSle=C+1;  RHSre=RE-1; }
  else {
    if (C<LE) {
      LHSle=0,LHSre=-1;
      RHSle=LE;  RHSre=RE-1; }
    else {//RE<C
      LHSle=LE; LHSre=RE-1;
      RHSle=0,RHSre=-1; }
  }

  for (g=0;g<numgen;g++) {

    if (C<LE) {
      //make column(s) C
      for (i=0;i<7;i++) { alpha(7*g+i,C)=1; }
      
      //make column LE
      for (j=0;j<7;j++) {

	sum=0;
	if (j<4) tsj=0; else tsj=3;//needed to get transstore entry
	for (i=0;i<7;i++) {

	  sum+=
	    alpha(7*g+i,C)*transstore(4*i+(j-tsj),C);
	}
	alpha(7*g+j,LE)=sum; 
      }      
    }

    else {//(LE<C)
      //make column(s) LE
      for (i=0;i<7;i++) {
	if (i<4) tsi=0; else tsi=3;
	alpha(7*g+i,LE)=initstore(i-tsi,0)*obsprobs(13*g+i-tsi,LE); 

	//delete me
	//if (g==7) cout << "hl " << i << " " << alpha(7*g+i,LE) << " "
	//       << initstore(i-tsi,0) << " "
	//       << obsprobs(13*g+i-tsi,LE) << endl;
      }
    }

    //make column(s) LE+1:C-1 or LE+1:RE (updated)
    for (t=LHSle;t<=LHSre;t++) {
      for (j=0;j<7;j++) {

	sum=0;
	for (i=0;i<7;i++) {

	  if (j<4) tsj=0; else tsj=3;
	  if (i<4) { tsi=0; indi=1; } 
	  else {
	    tsi=3;
	    if (j==0) indi=1; 
	    else indi=0; }
	  
	  sum+=
	    alpha(7*g+i,t)*transstore(7*(i-tsi)+j,t)*
	    obsprobs(13*g+ j -6*indi +6,t+1);

	}
	alpha(7*g+j,t+1)=sum; 
      } 
    }
    
    if (LE<C && C<RE) {

      /*
      //make column(s) C 

      //j=AA
      sum=0;
      for (i=0;i<7;i++) {
	if (i<4) tsi=0; else tsi=3;
	sum+=alpha(7*g+i,C-1)*transstore((i-tsi)*7  ,C-1); }
      if (sum>0) sum/=initstore(0,1);
      alpha(7*g  ,C)=sum;

      //j=AN1/AN0
      for (indj=1;indj>=0;indj--) {
	//make numerator
	sum=0.0;
	prod1=2.0;
	prod2=2.0;
	if (C+1<RE) {
	  prod1*= ( transstoreC( (2-indj)*4+(2-indj),3)*
		    obsCnum[1][28*g+0+14*(1-indj) ]+
		    transstoreC( (2-indj)*4+(3     ),3)*
		    obsCnum[1][28*g+1+14*(1-indj)]+ 
		    transstoreC( (5-indj)*4+(1+indj),3)*
		    obsCnum[1][28*g+3+14*(1-indj)]+
		    transstoreC( (5-indj)*4+(3     ),3)*
		    obsCnum[1][28*g+4+14*(1-indj)] );
	  prod2*= ( transstoreC( (3     )*4+(3     ),3)*
		    obsCnum[1][28*g+2+14*(1-indj)] +
		    transstoreC( (6     )*4+(3     ),3)*
		    obsCnum[1][28*g+5+14*(1-indj)] ); }
	
	for (indi=1;indi>=0;indi--) {

	  sum+=
	    ( alpha(7*g+(1+(1-indi)*3),C-1)*
	      transstore(7*(1           )+(1           ),C-1) +
	      alpha(7*g+(3+(1-indi)*3),C-1)*
	      transstore(7*(3           )+(1           ),C-1) )*
	    ( transstoreC(4*(1+3*(1-indj))+(2-indj),2)*
	      obsCnum[0][20*g+ 0+2*(1-indi)+10*(1-indj) ]*prod1+
	      transstoreC(4*(1+3*(1-indj))+(     3),2)*
	      obsCnum[0][20*g+ 1+2*(1-indi)+10*(1-indj) ]*prod2 );

	}

	if (sum>0) {
	  //make denominator
	  prod1=2.0;
	  prod2=2.0;

	  if (C+1<RE) {
	    prod1*= (transstoreC((2-indj)*4+(2-indj),3)*
		     obsCden[1][10*g+(0+2*(1-indj)) ]+
		     transstoreC((2-indj)*4+(3     ),3)*
		     obsCden[1][10*g+(1+2*(1-indj)) ]+
		     transstoreC((5-indj)*4+(1+indj),3)*
		     obsCden[1][10*g+(5+2*(1-indj)) ]+
		     transstoreC((5-indj)*4+(3     ),3)*
		     obsCden[1][10*g+(6+2*(1-indj)) ] );
	    prod2*= (transstoreC(3*4+3,3)*obsCden[1][10*g+4] +
		     transstoreC(6*4+3,3)*obsCden[1][10*g+9] );
	  }
	  
	  sum/=initstore(1,1)*( transstoreC((1+3*(1-indj))*4+(2-indj),2)*
				obsCden[0][3*g+(0+(1-indj))]*prod1+
				transstoreC((1+3*(1-indj))*4+(3         ),2)*
				obsCden[0][3*g+(2         )]*prod2 );
		
	}
	alpha(7*g+(1+3*(1-indj)),C)=sum;

      }
      
      //j=NA1/NA0
      for (indj=1;indj>=0;indj--) {
	//make numerator
	sum=0.0;
	prod1=2.0;
	prod2=2.0;
	if (C+1<RE) {
	  prod1*= ( transstoreC( (1+indj)*4+(1+indj),3)*
		    obsCnum[1][28*g+6+14*(1-indj) ]+
		    transstoreC( (1+indj)*4+(3     ),3)*
		    obsCnum[1][28*g+7+14*(1-indj)]+ 
		    transstoreC( (4+indj)*4+(2-indj),3)*
		    obsCnum[1][28*g+9+14*(1-indj)]+
		    transstoreC( (4+indj)*4+(3     ),3)*
		    obsCnum[1][28*g+10+14*(1-indj)] );
	  prod2*= ( transstoreC( (3     )*4+(3     ),3)*
		    obsCnum[1][28*g+8+14*(1-indj)] +
		    transstoreC( (6     )*4+(3     ),3)*
		    obsCnum[1][28*g+11+14*(1-indj)] ); }
	
	for (indi=1;indi>=0;indi--) {
	  sum+=
	    ( alpha(7*g+(2+(1-indi)*3),C-1)*
	      transstore(7*(2           )+(2           ),C-1) +
	      alpha(7*g+(3+(1-indi)*3),C-1)*
	      transstore(7*(3           )+(2           ),C-1) )*
	    ( transstoreC(4*(2+3*(1-indj))+(1+indj),2)*
	      obsCnum[0][20*g+ 4+2*(1-indi)+10*(1-indj) ]*prod1+
	      transstoreC(4*(2+3*(1-indj))+(     3),2)*
	      obsCnum[0][20*g+ 5+2*(1-indi)+10*(1-indj) ]*prod2 );

	}

	if (sum>0) {
	  //make denominator
	  prod1=2.0;
	  prod2=2.0;
	  
	  if (C+1<RE) {
	    prod1*= (transstoreC((1+indj)*4+(1+indj),3)*
		     obsCden[1][10*g+(0+2*(  indj)) ]+
		     transstoreC((1+indj)*4+(3     ),3)*
		     obsCden[1][10*g+(1+2*(  indj)) ]+
		     transstoreC((4+indj)*4+(2-indj),3)*
		     obsCden[1][10*g+(5+2*(  indj)) ]+
		     transstoreC((4+indj)*4+(3     ),3)*
		     obsCden[1][10*g+(6+2*(  indj)) ] );
	    prod2*= (transstoreC(3*4+3,3)*obsCden[1][10*g+4] +
		     transstoreC(6*4+3,3)*obsCden[1][10*g+9] );
	  }
	  
	  sum/=initstore(2,1)*( transstoreC((2+3*(1-indj))*4+(1+indj),2)*
				obsCden[0][3*g+ indj]*prod1+
				transstoreC((2+3*(1-indj))*4+(3     ),2)*
				obsCden[0][3*g+ 2   ]*prod2 );
	}
	
	alpha(7*g+(2+3*(1-indj)),C)=sum;

      }
      
      //j=NN1/NN0
      for (indj=1;indj>=0;indj--) {
	//make numerator
	sum=0.0;
	prod1=2.0;
	prod2=2.0;
	if (C+1<RE) {
	  prod2*= ( transstoreC( (3     )*4+(3     ),3)*
		    obsCnum[1][28*g+12+14*(1-indj)] +
		    transstoreC( (6     )*4+(3     ),3)*
		    obsCnum[1][28*g+13+14*(1-indj)] ); }
	
	for (indi=1;indi>=0;indi--) {
	  sum+=
	    ( alpha(7*g+(3+(1-indi)*3),C-1)*
	      transstore(7*(3           )+(3           ),C-1) )*
	    ( transstoreC(4*(3+3*(1-indj))+(     3),2)*
	      obsCnum[0][20*g+ 9-indi +10*(1-indj) ]*prod2 );

	}
	
	if (sum>0) {
	  //make denominator
	  prod2=2.0;
	  
	  if (C+1<RE) {
	    prod2*= (transstoreC(3*4+3,3)*obsCden[1][10*g+4] +
		     transstoreC(6*4+3,3)*obsCden[1][10*g+9] );
	  }
	  
	  sum/=initstore(3,1)*( transstoreC((3+3*(1-indj))*4+(3         ),2)*
				obsCden[0][3*g+(2         )]*prod2 );

	}
	alpha(7*g+(3+3*(1-indj)),C)=sum;

      }
      */

      //make column C+1
      //k=0 (AA)
      sum=0.0;
      for (i=0;i<7;i++) {
	if (i<4) tsi=0; else tsi=3;
	sum+=alpha(7*g+i,C-1)*transstore(7*(i-tsi)  ,C-1); 
	
      }
      sum*=transstoreC(0,2)*obsprobs(13*g,C+1);
      if (sum>1e-250) sum/=initstore(0,4)*obsprobs(13*g,C+1);
      alpha(7*g  ,C+1)=sum;
      
      //k=1 (AN1) and k=4 (AN0)
      for (indk=1;indk>=0;indk--) {
	sum=0.0;

	//add q0=AA
	for (i=0;i<7;i++) {
	  if (i<4) tsi=0; else tsi=3;
	  sum+=alpha(7*g+i,C-1)*transstore(7*(i-tsi)  ,C-1); 
	}
	sum*=transstoreC(4*0+1,2)*obsCden[0][3*g  ];
	if (RE>C+1)
	  sum*=
	    2*transstoreC((1+(1-indk)*3)*4+(2-indk),3)*
	    obsCden[1][10*g+5*(1-indk)  ]+
	    2*transstoreC((1+(1-indk)*3)*4+(3     ),3)*
	    obsCden[1][10*g+5*(1-indk)+1];

	//now add q0=AN1,NA0
	for (indj=1;indj>=0;indj--) {	
	  if (RE==C+1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((1+3*(1-indk))*4+(2-indk),3)*
	      obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+2]+
	      2*transstoreC((1+3*(1-indk))*4+(3     ),3)*
	      obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+3]; 

	  }

	  for (indi=1;indi>=0;indi--) {
	    sum+=
	      (alpha(7*g+ (2-indj)+3*(1-indi),C-1)*
	       transstore((2-indj)*7+((2-indj)+(1-indj)*3),C-1)+
	       alpha(7*g+ (3     )+3*(1-indi),C-1)*
	       transstore((3     )*7+((2-indj)+(1-indj)*3),C-1))*
	      transstoreC(4*((2-indj)+(1-indj)*3) + 1,2)*
	      obsCnum[4][23*g+(1-indj)*2+(1-indi)+1]*prod2; 

	  }
	}
            
	if (sum>0) {
	  //create denominator
	  sum/=initstore(1,4)*obsCden[0][3*g  ];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((1+(1-indk)*3)*4+(2-indk),3)*
	      obsCden[1][10*g+5*(1-indk)  ]+
	      2*transstoreC((1+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+1];
	} 
	alpha(7*g+3*(1-indk)+1,C+1)=sum;
      }

      //k=2 (NA1) and k=5 (NA0)
      for (indk=1;indk>=0;indk--) {
	sum=0.0;

	//add q0=AA
	for (i=0;i<7;i++) {
	  if (i<4) tsi=0; else tsi=3;
	  sum+=alpha(7*g+i,C-1)*transstore(7*(i-tsi)  ,C-1); 
	}
	sum*=transstoreC(4*0+2,2)*obsCden[0][3*g+1];
	if (RE>C+1)
	  sum*=
	    2*transstoreC((2+(1-indk)*3)*4+(1+indk),3)*
	    obsCden[1][10*g+5*(1-indk)+2]+
	    2*transstoreC((2+(1-indk)*3)*4+(3     ),3)*
	    obsCden[1][10*g+5*(1-indk)+3];

	//now add q0=NA1,AN0
	for (indj=1;indj>=0;indj--) {	
	  if (RE==C+1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((2+3*(1-indk))*4+(1+indk),3)*
	      obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+8]+
	      2*transstoreC((2+3*(1-indk))*4+(3     ),3)*
	      obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+9]; }
	  
	  for (indi=1;indi>=0;indi--) {
	    sum+=
	      (alpha(7*g+ (1+indj)+3*(1-indi),C-1)*
	       transstore((1+indj)*7+((1+indj)+(1-indj)*3),C-1)+
	       alpha(7*g+ (3     )+3*(1-indi),C-1)*
	       transstore((3     )*7+((1+indj)+(1-indj)*3),C-1))*
	      transstoreC(4*((1+indj)+(1-indj)*3) + 2,2)*
	      obsCnum[4][23*g+(1-indj)*2+(1-indi)+6]*prod2; 

	  }
	}
	
	if (sum>0) {
	  //create denominator
	  sum/=initstore(2,4)*obsCden[0][3*g+1];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((2+(1-indk)*3)*4+(1+indk),3)*
	      obsCden[1][10*g+5*(1-indk)+2]+
	      2*transstoreC((2+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+3];

	}
	alpha(7*g+3*(1-indk)+2,C+1)=sum;
      }
      
      //k=3 (NN1) or k=6 (NN0)
      for (indk=1;indk>=0;indk--) {
	sum=0.0;
	
	//add q0=AA
	for (i=0;i<7;i++) {
	  if (i<4) tsi=0; else tsi=3;
	  sum+=alpha(7*g+i,C-1)*transstore(7*(i-tsi)  ,C-1); }
	sum*=transstoreC(4*0+3,2)*obsCden[0][3*g+2];
	if (RE>C+1)
	  sum*=
	    2*transstoreC((3+(1-indk)*3)*4+(3    ),3)*
	    obsCden[1][10*g+5*(1-indk)+4];

	//now add q0=AN1,AN0
	for (indj=1;indj>=0;indj--) {	
	  if (RE==C+1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	      obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+13]; }

	  for (indi=1;indi>=0;indi--) {
	    sum+=
	      (alpha(7*g+ (1     )+3*(1-indi),C-1)*
	       transstore((1     )*7+((1     )+(1-indj)*3),C-1)+
	       alpha(7*g+ (3     )+3*(1-indi),C-1)*
	       transstore((3     )*7+((1     )+(1-indj)*3),C-1))*
	      transstoreC(4*((1     )+(1-indj)*3) + 3,2)*
	      obsCnum[4][23*g+(1-indj)*6+(1-indi)+11]*prod2; }
	}

	//now add q0=NA1,NA0
	for (indj=1;indj>=0;indj--) {	
	  if (RE==C+1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	      obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+14]; }

	  for (indi=1;indi>=0;indi--) {
	    sum+=
	      (alpha(7*g+ (2     )+3*(1-indi),C-1)*
	       transstore((2     )*7+((2     )+(1-indj)*3),C-1)+
	       alpha(7*g+ (3     )+3*(1-indi),C-1)*
	       transstore((3     )*7+((2     )+(1-indj)*3),C-1))*
	      transstoreC(4*((2     )+(1-indj)*3) +3,2)*
	       obsCnum[4][23*g+(1-indj)*6+(1-indi)+13]*prod2; }
	}

	//now add q0=NN1,NN0
	for (indj=1;indj>=0;indj--) {	
	  if (RE==C+1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	      obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+15]; 
	    
	  }

	  for (indi=1;indi>=0;indi--) {
	    sum+=
	      (alpha(7*g+ (3     )+3*(1-indi),C-1)*
	       transstore((3     )*7+(3     ),C-1))*
	      transstoreC(4*(3      )  +3,2)*
	      obsCnum[4][23*g+(1-indj)*6+(1-indi)+15]*prod2; 

	  }
	}

	if (sum>0) {
	  //create denominator
	  sum/=initstore(3,4)*obsCden[0][3*g+2];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((3+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+4];

	}
	alpha(7*g+3*(1-indk)+3,C+1)=sum;

      }      
    }
  
    //make column(s) C+2:RE or LE+1:RE (updated)
    for (t=RHSle;t<=RHSre;t++) {
      for (j=0;j<7;j++) {

	sum=0;
	for (i=0;i<7;i++) {

	  if (i<4) indi=1; else indi=0;
	  if (j<4) { indj=1; tsj=0; } 
	  else {
	    tsj=3; 
	    if (i==0) indj=1;
	    else indj=0; }

	  sum+=
	    alpha(7*g+i,t)*transstore(4*i+(j-tsj),t)*
	    obsprobs(13*g+ i- 3*indi -3*indj +6,t); 

	}
	alpha(7*g+j,t+1)=sum; 
      }
    }

    if (RE<C) {
      //make column C 
      for (j=0;j<7;j++) {

	sum=0;
	if (j<4) tsj=0; else tsj=3;//needed to get transstore entry
	for (i=0;i<7;i++) {
	  if (i<4) tsi=0; else tsi=3;
	  
	  sum+=
	    alpha(7*g+i,RE)*transstore(7*(i-tsi)+j,RE); 
	}
	if (sum>0) sum/=initstore(j-tsj,1);

	alpha(7*g+j,C)=sum; 

      }
    }

  }
  
  //check that all entries are between 0 and 1
  for (i=0;i<alpha.dimx();i++) {
    if (C<LE || RE<C) { 
      if ( !(-0.00001<alpha(i,C) && alpha(i,C)<1.00001) ) {
	errorfs << "ERROR: alpha(" << i << "," << C << ") = " 
	     << alpha(i,C) << endl;
	exit(1);
      }
    }
    for (j=LE;j<=RE;j++) {
      if ( !(-0.00001<alpha(i,j) && alpha(i,j)<1.00001) ) {
	errorfs << "ERROR: alpha(" << i << "," << j << ") = " 
	     << alpha(i,j) << endl;
	exit(1);
      } 
    }
  }
  
}

void makebetagenmkv2(dgrid &beta, const dgrid &initstore, 
		     const dgrid &transstore, const dgrid &transstoreC,
		     const dgrid &obsprobs, const dvectors &obsCnum,
		     const dvectors &obsCden, dvector &scratch,
		     const int &C, const int &LE, const int &RE)
{

  //makes beta object, assuming genotype data for affecteds

  int i,j,k,g,t;
  const int numgen=beta.dimx()/7;
  int LHSle, LHSre, RHSle, RHSre,tsi,tsj,tsk,indi,indj,indk;
  double sum,prod2;

  //these are endpoints for t, rather than t+1
  if (LE<C && C<RE) {
    LHSle=LE; LHSre=C-2;
    RHSle=C+1;  RHSre=RE-1; }
  else {
    if (C<LE) {
      LHSle=0,LHSre=-1;
      RHSle=LE;  RHSre=RE-1; }
    else {//RE<C
      LHSle=LE; LHSre=RE-1;
      RHSle=0,RHSre=-1; }
  }

  for (g=0;g<numgen;g++) {

    if (RE<C) {
      //make column(s) C
      for (i=0;i<4;i++) {
	beta(7*g+i,C)=initstore(i,1); }
      for (i=4;i<7;i++) {
	beta(7*g+i,C)=beta(7*g+i-3,C); }

      //make column RE
      for (i=0;i<7;i++) {
	beta(7*g+i,RE)=1.0; }

    }
    else {//C<RE
      //make column(s) RE
      for (i=0;i<4;i++) {
	beta(7*g+i,RE)= initstore(i,2)*obsprobs(13*g+i,RE); 
      }
      for (i=4;i<7;i++) beta(7*g+i,RE)=beta(7*g+i-3,RE);
    }
    

    //make column(s) RE-1:C or RE-1:LE (updated)
    for (t=RHSre;t>=RHSle;t--) {
      for (i=0;i<7;i++) {
	sum=0;

	for (j=0;j<7;j++) {
	  if (i<4) indi=1; else indi=0;
	  if (j<4) { indj=1; tsj=0; } 
	  else {
	    tsj=3; 
	    if (i==0) indj=1;
	    else indj=0; }

	  sum+=
	    beta(7*g+j,t+1)*transstore(4*i+(j-tsj),t)*
	    obsprobs(13*g+ i- 3*indi -3*indj +6,t); 

	}
	beta(7*g+i,t)=sum; } }

      
    if (LE<C && C<RE) {
      //make column(s) C-1 (see notes for explanation about why this is messy)

      //i=0 (AA)
      sum=0.0;
      for (k=0;k<7;k++) {
	if (k>3) tsk=3; else tsk=0;
	sum+=beta(7*g+k,C+1)*transstore(0*4+ k-tsk,C); }
      sum*=transstoreC(0,1)*obsprobs(13*g,C-1);
      if (sum>1e-200) sum/=initstore(0,3)*obsprobs(13*g,C-1);
      beta(7*g  ,C-1)=sum;

      //i=1 (AN1) and i=4 (AN0)
      for (indi=1;indi>=0;indi--) {
	sum=0;

	//add q0=AA
	for (k=0;k<7;k++) {
	  if (k<4) tsk=0; else tsk=3;
	  sum+=beta(7*g+k,C+1)*transstore(4*0+(k-tsk),C); 
	}
	sum*=transstoreC(7*1  ,1)*obsCden[2][3*g  ];
	if (LE<C-1)
	  sum*=
	    2*transstoreC((2-indi)*7+1+(1-indi)*3,0)*
	    obsCden[3][10*g+5*(1-indi)  ]+
	    2*transstoreC((3     )*7+1+(1-indi)*3,0)*
	    obsCden[3][10*g+5*(1-indi)+1];

	//now add q0=AN1,AN0
	for (indj=1;indj>=0;indj--) {	
	  if (LE==C-1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((2-indi)*7+(1+3*(1-indi)),0)*
	      obsCnum[3][38*g+19*(1-indi)+2*(1-indj)+2]+
	      2*transstoreC((3     )*7+(1+3*(1-indi)),0)*
	      obsCnum[3][38*g+19*(1-indi)+2*(1-indj)+3]; }

	  for (indk=1;indk>=0;indk--) {
	    sum+=
	      (beta(7*g+ (2-indj)+3*(1-indk),C+1)*
	       transstore((1+(1-indj)*3)*4+(2-indj),C)+
	       beta(7*g+ (3     )+3*(1-indk),C+1)*
	       transstore((1+(1-indj)*3)*4+(3     ),C))*
	      transstoreC(7*1+ 1+(1-indj)*3,1)*
	      obsCnum[2][23*g+(1-indj)*2+(1-indk)+1]*prod2; 
	    
	  }
	}

	//delete me
	//if (g==0 && indi==1) 
	//cout << "beta " << LE << " " << C << " " << RE << " "
	//     << sum << " "
	//     << initstore(1,3)*obsCden[2][3*g  ] << " ";
	       
	//create denominator
	if (sum>1e-200) {
	  //if (g==0 && indi==1) cout << sum << " ";//delete me
	  sum/=initstore(1,3)*obsCden[2][3*g  ];
	  //if (g==0 && indi==1) cout << sum << endl;//delete me
	  if(LE<C-1)
	    sum/=
	      2*transstoreC((2-indi)*7+1+(1-indi)*3,0)*
	      obsCden[3][10*g+5*(1-indi)  ]+
	      2*transstoreC((3     )*7+1+(1-indi)*3,0)*
	      obsCden[3][10*g+5*(1-indi)+1];
	  
	}
	beta(7*g+3*(1-indi)+1,C-1)=sum;

      }

      //i=2 (NA1) and i=5 (NA0)
      for (indi=1;indi>=0;indi--) {
	sum=0;
	
	//add q0=AA
	for (k=0;k<7;k++) {
	  if (k<4) tsk=0; else tsk=3;
	  sum+=beta(7*g+k,C+1)*transstore(4*0+(k-tsk),C); 
	}
	sum*=transstoreC(7*2  ,1)*obsCden[2][3*g+1];
	if (LE<C-1)
	  sum*=
	    2*transstoreC((1+indi)*7+2+(1-indi)*3,0)*
	    obsCden[3][10*g+5*(1-indi)+2]+
	    2*transstoreC((3     )*7+2+(1-indi)*3,0)*
	    obsCden[3][10*g+5*(1-indi)+3];
	
	//now add NA1,NA0
	for (indj=1;indj>=0;indj--) {	
	  if (LE==C-1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((1+indi)*7+(2+3*(1-indi)),0)*
	      obsCnum[3][38*g+19*(1-indi)+2*(1-indj)+8]+
	      2*transstoreC((3     )*7+(2+3*(1-indi)),0)*
	      obsCnum[3][38*g+19*(1-indi)+2*(1-indj)+9]; }

	  for (indk=1;indk>=0;indk--) {
	    sum+=
	      (beta(7*g+ (1+indj)+3*(1-indk),C+1)*
	       transstore((2+(1-indj)*3)*4+(1+indj),C)+
	       beta(7*g+ (3     )+3*(1-indk),C+1)*
	       transstore((2+(1-indj)*3)*4+(3     ),C))*
	      transstoreC(7*2+ 2+(1-indj)*3,1)*
	      obsCnum[2][23*g+(1-indj)*2+(1-indk)+6]*prod2; 
	  }
	}

	//create denominator
	if (sum>1e-200) {
	  sum/=initstore(2,3)*obsCden[2][3*g+1];
	  if(LE<C-1)
	    sum/=
	      2*transstoreC((1+indi)*7+2+(1-indi)*3,0)*
	      obsCden[3][10*g+5*(1-indi)+2]+
	      2*transstoreC((3     )*7+2+(1-indi)*3,0)*
	      obsCden[3][10*g+5*(1-indi)+3];

	}
	beta(7*g+3*(1-indi)+2,C-1)=sum;

      }

      //i=3 (NN1) or i=6 (NN0)
      for (indi=1;indi>=0;indi--) {
	sum=0;
	
	//add q0=AA
	for (k=0;k<7;k++) {
	  if (k<4) tsk=0; else tsk=3;
	  sum+=beta(7*g+k,C+1)*transstore(4*0+(k-tsk),C); 
	}
	sum*=transstoreC(7*3  ,1)*obsCden[2][3*g+2];
	if (LE<C-1)
	  sum*=
	    2*transstoreC(3*7+3+(1-indi)*3,0)*obsCden[3][10*g+5*(1-indi)+4];

	//add q0=1,4
	for (indj=1;indj>=0;indj--) {	
	  if (LE==C-1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((3     )*7+(3           ),0)*
	      obsCnum[3][38*g+19*(1-indi)+3*(1-indj)+13]; }

	  for (indk=1;indk>=0;indk--) {
	    sum+=
	      (beta(7*g+ (2-indj)+3*(1-indk),C+1)*
	       transstore((1+(1-indj)*3)*4+(2-indj),C)+
	       beta(7*g+ (3     )+3*(1-indk),C+1)*
	       transstore((1+(1-indj)*3)*4+(3     ),C))*
	      transstoreC(7*3+ 1+(1-indj)*3,1)*
	      obsCnum[2][23*g+(1-indj)*6+(1-indk)+11]*prod2; 
	  }
	}

	//add q1=2,5
	for (indj=1;indj>=0;indj--) {	
	  if (LE==C-1) prod2=1.0;
	  else {
	    prod2=
	      2*transstoreC((3     )*7+(3           ),0)*
	      obsCnum[3][38*g+19*(1-indi)+3*(1-indj)+14]; }

	  for (indk=1;indk>=0;indk--) {
	    sum+=
	      (beta(7*g+ (1+indj)+3*(1-indk),C+1)*
	       transstore((2+(1-indj)*3)*4+(1+indj),C)+
	       beta(7*g+ (3     )+3*(1-indk),C+1)*
	       transstore((2+(1-indj)*3)*4+(3     ),C))*
	      transstoreC(7*3+ 2+(1-indj)*3,1)*
	      obsCnum[2][23*g+(1-indj)*6+(1-indk)+13]*prod2; 
	  }
	}

	//add q=3,6
	for (indj=1;indj>=0;indj--) {	
	  if (LE==C-1) prod2=1.0;
	  else 
	    prod2=obsCnum[3][38*g+19*(1-indi)+3*(1-indj)+15]; 

	  for (indk=1;indk>=0;indk--) {
	    sum+=
	      (beta(7*g+ (3     )+3*(1-indk),C+1)*
	       transstore((3     )*4+(3     ),C))*
	      transstoreC(7*3  +3,1)*
	      obsCnum[2][23*g+(1-indj)*6+(1-indk)+15]*prod2; 

	  }
	}

	//create denominator
	if (sum>1e-200) {
	  sum/=initstore(3,3)*obsCden[2][3*g+2];
	  if(LE<C-1)
	    sum/=
	      2*transstoreC(3*7+3+(1-indi)*3,0)*obsCden[3][10*g+5*(1-indi)+4];
	  
	}

	beta(7*g+3*(1-indi)+3,C-1)=sum;

      }
    }

    //make column(s) C-2:LE or RE-1:LE (updated)
    for (t=LHSre;t>=LHSle;t--) {
      for (i=0;i<7 ;i++) {

	sum=0;
	for (j=0;j<7;j++) {

	  if (j<4) tsj=0; else tsj=3;
	  if (i<4) { tsi=0; indi=1; } 
	  else {
	    tsi=3;
	    if (j==0) indi=1; 
	    else indi=0; }
	    
	  sum+=
	    beta(7*g+j,t+1)*transstore(7*(i-tsi)+j,t)*
	    obsprobs(13*g+ j -6*indi +6,t+1); 

	}
	beta(7*g+i,t)=sum; 


      }
    }
  
    if (C<LE) {
      //make column(s) C
      for (i=0;i<7;i++) {
	sum=0;
	if (i<4) tsi=0; else tsi=3;

	for (j=0;j<7;j++) {
	  if (j<4) { tsj=0; indj=0;}
	  else { if (i==0) indj=0; else indj=3; tsj=3; }
	  
	  sum+=
	    beta(7*g+j,LE)*transstore(4*(i-tsi)+(j-tsj),C);

	  
	}
	beta(7*g+i,C)=sum; } }
  }

    for (i=0;i<beta.dimx();i++) {
      if (C<LE || RE<C) {
	if ( !(-0.0001<beta(i,C) && beta(i,C)<1.0001) ) {
	  errorfs << "ERROR: beta(" << i << "," << C << ") = " 
		  << beta(i,C) << endl;
	  exit(1);
	}
      }
      for (j=LE;j<=RE;j++)
	if ( !(-0.0001<beta(i,j) && beta(i,j)<1.0001) ) {
	  errorfs << "ERROR: beta(" << i << "," << j << ") = " 
		  << beta(i,j) << endl;
	  exit(1);
	}
    }

}

void makegammaC(dvectors &gammaC, const dgrid &alpha, const dgrid &beta,
		const dgrid &obsprobs, const dvectors &obsCnum,
		const dvectors &obsCden, const dgrid &initstore,
		const dgrid &transstore,
		const dgrid &transstoreC, const int &C, const int &LE,
		const int &RE)
{
  int g,i,j,k,indi,indj,indk,tsi,tsk;
  const int numgen=alpha.dimx()/7;
  double prod2,sum=0;

  if (LE<C && C<RE) {
  for (g=0;g<numgen;g++) {

    //make gammaC[1]
    for (k=0;k<7;k++) {
      
      for (j=0;j<7;j++) {
	
	if (k<4) tsk=0; else tsk=3;
	
	gammaC[1][49*g+7*j+k]=
	  beta(7*g+k,C+1)*transstore(4*j+(k-tsk),C); } }
    
    //make gammaC[0]   [code from alpha(,C+1)]
    
    //make k=0:6,j=AA
    sum=0.0;
    for (i=0;i<7;i++) {
      if (i<4) tsi=0; else tsi=3;
      sum+=alpha(7*g+i,C-1)*transstore(7*(i-tsi)  ,C-1); }
    
    //k=0
    gammaC[0][49*g      ]=sum;
    gammaC[0][49*g      ]*=transstoreC(0,2)*obsprobs(13*g,C+1);
    if (gammaC[0][49*g      ]>1e-200) 
      gammaC[0][49*g      ]/=
	initstore(0,1)*transstoreC(0,2)*obsprobs(13*g,C+1);

    //k=1:6
    for (k=1;k<7;k++) {
      if (k<4) tsk=0; else tsk=3;

      /*
      gammaC[0][49*g    +k]=sum;
      gammaC[0][49*g    +k]*=transstoreC(0*4+k-tsk,2)*obsCden[0][3*g+k-tsk-1];
      if (gammaC[0][49*g    +k]>0) 
	gammaC[0][49*g    +k]/=
	  initstore(0,1)*transstoreC(0*4+k-tsk,2)*obsCden[0][3*g+k-tsk-1]; 
      */
  
      gammaC[0][49*g    +k]=sum;
      if (sum>1e-200 && transstoreC(0*4+k-tsk,2)>0 
	  && obsCden[0][3*g+k-tsk-1]>0) 
	gammaC[0][49*g    +k]/=initstore(0,1);
      else gammaC[0][49*g    +k]=0;

    }

    //k=1,4

    for (indk=1;indk>=0;indk--) {
      for (indj=1;indj>=0;indj--) {	

	//k=1,5
	sum=0.0;
	
	if (RE==C+1) prod2=1.0;
	else {
	  prod2=
	    2*transstoreC((1+3*(1-indk))*4+(2-indk),3)*
	    obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+2]+
	    2*transstoreC((1+3*(1-indk))*4+(3     ),3)*
	    obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+3]; }

	for (indi=1;indi>=0;indi--) {
	  sum+=
	    (alpha(7*g+ (2-indj)+3*(1-indi),C-1)*
	     transstore((2-indj)*7+((2-indj)+(1-indj)*3),C-1)+
	     alpha(7*g+ (3     )+3*(1-indi),C-1)*
	     transstore((3     )*7+((2-indj)+(1-indj)*3),C-1))*
	    transstoreC(4*((2-indj)+(1-indj)*3) + 1,2)*
	    obsCnum[4][23*g+(1-indj)*2+(1-indi)+1]*prod2; }

	if (sum>0) {
	  //create denominator
	  sum/=initstore(2-indj,1)*
	    transstoreC( ((2-indj)+(1-indj)*3)*4+ 1,2)*
	    obsCden[0][3*g  ];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((1+(1-indk)*3)*4+(2-indk),3)*
	      obsCden[1][10*g+5*(1-indk)  ]+
	      2*transstoreC((1+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+1]; }

	gammaC[0][49*g+ ( (2-indj)+(1-indj)*3)*7  +1+(1-indk)*3]=sum; 

      }
    }
    
    //k=2,5
    for (indj=1;indj>=0;indj--) {	
      for (indk=1;indk>=0;indk--) {
	
	//k=2,4
	sum=0.0;
	
	if (RE==C+1) prod2=1.0;
	else {
	  prod2=
	    2*transstoreC((2+3*(1-indk))*4+(1+indk),3)*
	    obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+8]+
	    2*transstoreC((2+3*(1-indk))*4+(3     ),3)*
	    obsCnum[5][38*g+19*(1-indk)+2*(1-indj)+9]; }
	
	for (indi=1;indi>=0;indi--) {
	  sum+=
	    (alpha(7*g+ (1+indj)+3*(1-indi),C-1)*
	     transstore((1+indj)*7+((1+indj)+(1-indj)*3),C-1)+
	     alpha(7*g+ (3     )+3*(1-indi),C-1)*
	     transstore((3     )*7+((1+indj)+(1-indj)*3),C-1))*
	    transstoreC(4*((1+indj)+(1-indj)*3) + 2,2)*
	    obsCnum[4][23*g+(1-indj)*2+(1-indi)+6]*prod2; }

	if (sum>0) {
	  //create denominator
	  sum/=initstore(1+indj,1)*
	    transstoreC( ((1+indj)+(1-indj)*3)*4+ 2,2)*
	    obsCden[0][3*g+1];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((2+(1-indk)*3)*4+(1+indk),3)*
	      obsCden[1][10*g+5*(1-indk)+2]+
	      2*transstoreC((2+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+3]; }

	gammaC[0][49*g+ ( (1+indj)+(1-indj)*3)*7  +2+(1-indk)*3]=sum; 

      }
    }
    
    //k=3,6
    for (indj=1;indj>=0;indj--) {	
      for (indk=1;indk>=0;indk--) {
	
	//j=1,4 
	sum=0.0;

	if (RE==C+1) prod2=1.0;
	else {
	  prod2=
	    2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	    obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+13]; }

	for (indi=1;indi>=0;indi--) {
	  sum+=
	    (alpha(7*g+ (1     )+3*(1-indi),C-1)*
	     transstore((1     )*7+((1     )+(1-indj)*3),C-1)+
	     alpha(7*g+ (3     )+3*(1-indi),C-1)*
	     transstore((3     )*7+((1     )+(1-indj)*3),C-1))*
	    transstoreC(4*((1     )+(1-indj)*3) + 3,2)*
	    obsCnum[4][23*g+(1-indj)*6+(1-indi)+11]*prod2; 
	
	  //if (g==7 && indj==0 && indk==1)
	  //cout << "AN0 NN1 num " << indi << " " << sum << " "
	    //<< alpha(7*g+1+3*(1-indi),C-1) << " "
	    // << transstore((1)*7+((1)+(1-indj)*3),C-1) << " "
	    // << alpha(7*g+3+3*(1-indi),C-1) << " "
	    // << transstore((3)*7+((1)+(1-indj)*3),C-1) << " "
	    // << transstoreC(4*((1   )+(1-indj)*3)+3,2) << " "
	  //   << obsCnum[4][23*g+(1-indj)*6+(1-indi)+11] << " "
	    // << prod2 << " "
	  //   << (1-indj)*6+(1-indi)+11 << " "
	  //   << endl;
	}
	if (sum>0) {
	  //create denominator
	  sum/=initstore(1,1)*transstoreC( (1+(1-indj)*3)*4+ (3     ),2)*
	    obsCden[0][3*g+2];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((3+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+4]; }
	
	gammaC[0][49*g+ (1+(1-indj)*3)*7  +(3     )+(1-indk)*3]=sum; 

	sum=0.0;
	if (RE==C+1) prod2=1.0;
	else {
	  prod2=
	    2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	    obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+14]; }
	
	for (indi=1;indi>=0;indi--) {
	  sum+=
	    (alpha(7*g+ (2     )+3*(1-indi),C-1)*
	     transstore((2     )*7+((2     )+(1-indj)*3),C-1)+
	     alpha(7*g+ (3     )+3*(1-indi),C-1)*
	     transstore((3     )*7+((2     )+(1-indj)*3),C-1))*
	    transstoreC(4*((2     )+(1-indj)*3) +3,2)*
	    obsCnum[4][23*g+(1-indj)*6+(1-indi)+13]*prod2; }

	if (sum>0) {
	  //create denominator
	  sum/=initstore(2,1)*transstoreC( (2+(1-indj)*3)*4+ (3     ),2)*
	    obsCden[0][3*g+2];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((3+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+4];
	}
	
	gammaC[0][49*g+ (2+(1-indj)*3)*7  +(3     )+(1-indk)*3]=sum; 

	//k=3,6
	sum=0.0;
	if (RE==C+1) prod2=1.0;
	else {
	  prod2=
	    2*transstoreC((3+3*(1-indk))*4+(3     ),3)*
	    obsCnum[5][38*g+19*(1-indk)+3*(1-indj)+15]; 
	  
	}
	  
	for (indi=1;indi>=0;indi--) {
	  sum+=
	    (alpha(7*g+ (3     )+3*(1-indi),C-1)*
	     transstore((3     )*7+(3     ),C-1))*
	    transstoreC(4*(3      )  +3,2)*
	    obsCnum[4][23*g+(1-indj)*6+(1-indi)+15]*prod2; }
	
	
	if (sum>0) {
	  //create denominator
	  sum/=initstore(3,1)*transstoreC( (3+(1-indj)*3)*4+ (3     ),2)*
	    obsCden[0][3*g+2];
	  if (RE>C+1)
	    sum/=
	      2*transstoreC((3+(1-indk)*3)*4+(3     ),3)*
	      obsCden[1][10*g+5*(1-indk)+4];
	}
	
	gammaC[0][49*g+ (3+(1-indj)*3)*7  +(3     )+(1-indk)*3]=sum; 
	
      }
    }
    
  }

  //check that all entries are between 0 and 1
  for (g=0;g<numgen;g++) {
    for (i=0;i<2;i++) {
      
      for (j=0;j<7;j++) {
	for (k=0;k<7;k++) {
	
	  if ( !(-0.00001<gammaC[i][49*g+7*j+k] && 
		 gammaC[i][49*g+7*j+k]<1.00001) ) {
	    errorfs << "ERROR: gammaC[" << i << "][49*" << g << "+7*" 
		    << j << "+" << k << "] is " << gammaC[i][49*g+7*j+k]
		    << endl;
	    exit(1);
	  }
	}
      }
    }

  }

    
    
  }
}

